# -*- coding: utf-8 -*-
"""
Created on Fri May 29 10:43:49 2015

@author: Ferriss

Contains my most commonly used plotting style dictionaries (e.g., blue dots)
and some frequently used plotting setups, e.g., 3 subplots

"""
from __future__ import print_function, division, absolute_import
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.parasite_axes import SubplotHost
import numpy as np

style_points = {'color' : 'b', 'marker' : 'o', 'markersize' : 6,
                'fillstyle' : 'full', 'linestyle' : 'none',}
style_lightgreen = {'color' : 'lightgreen', 'linewidth' : 4}
style_blue = {'color' : 'blue', 'linestyle' : '-', 'marker': None}
style_points0 = {'color' : 'black', 'marker' : 'D', 'markersize' : 6,
                 'fillstyle' : 'full', 'linestyle' : 'none', 
                 'linewidth' : 1, 'alpha' : 1, 'label' : 'initial',
                 'markerfacecolor' : 'k'}
style_points1 = {'color' : 'red', 'marker' : '^', 'markersize' : 6,
                 'fillstyle' : 'full', 'linestyle' : 'none', 
                 'linewidth' : 1, 'alpha' : 1, 'label' : 'initial',
                 'markerfacecolor' : 'red'}
style_points2 = {'color' : 'indigo', 'marker' : 'o', 'markersize' : 6,
                 'fillstyle' : 'none', 'linestyle' : 'none', 
                 'linewidth' : 1, 'alpha' : 1, 'label' : 'initial',
                 'markerfacecolor' : 'w'}
style_points3 = {'color' : 'blue', 'marker' : 's', 'markersize' : 4,
                 'fillstyle' : 'none', 'linestyle' : 'none', 
                 'linewidth' : 1, 'alpha' : 1, 'label' : 'initial'}
style_points4 = {'color' : 'green', 'marker' : 'd', 'markersize' : 5,
                 'fillstyle' : 'none', 'linestyle' : 'none', 
                 'linewidth' : 1, 'alpha' : 1, 'label' : 'initial'}
style_points5 = {'color' : 'darkgoldenrod', 'marker' : 'D', 'markersize' : 4,
                 'fillstyle' : 'full', 'linestyle' : 'none', 
                 'linewidth' : 1, 'alpha' : 1, 'label' : 'initial',
                 'markerfacecolor' : 'y'}
style_points6 = {'color' : 'orangered', 'marker' : 'o', 'markersize' : 5,
                 'fillstyle' : 'none', 'linestyle' : 'none', 
                 'linewidth' : 1, 'alpha' : 1, 'label' : 'initial'}
style_points7 = {'color' : 'violet', 'marker' : 'v', 'markersize' : 7,
                 'fillstyle' : 'full', 'linestyle' : 'none', 
                 'linewidth' : 1, 'alpha' : 1, 'label' : 'initial'}
style_baseline = {'color' : 'k', 'linewidth' : 1, 'linestyle' :'-'}
style_spectrum = {'color' : 'b', 'linewidth' : 3}
style_spectrum_red = {'color' : 'r'}
style_fitpeak = {'color' : 'g', 'linewidth' : 1}
style_summed = {'color' : 'orangered', 'linewidth' : 2, 'linestyle' : '--'}
style_profile = {'markeredgecolor' : 'black', 'linestyle' : '', 'marker' : 'o', 
                 'markersize' : 10, 'markerfacecolor' : 'grey', 'alpha' : 0.5}
style_initial = {'color' : 'blue', 'label' : 'initial', 'linestyle' : '--'}                 
style_1a = {'linestyle' : '--', 'color' : 'k', 'marker' : None, 'linewidth' : 1}
style_1 = {'linestyle' : '-', 'color' : 'k', 'marker' : None, 'linewidth' : 1}
style_2 = {'color':'red', 'linewidth': 2.5, 'linestyle' : '-.'}
style_2a = {'color':'green', 'linewidth': 1.5, 'linestyle' : '--'}
style_3 = {'color':'orange', 'linewidth': 2., 'linestyle' : '--'}
style_3a = {'color':'blue', 'linewidth': 1., 'linestyle' : '--'}
style_4 = {'color':'yellow', 'linewidth':2.5, 'linestyle' : '-'}
style_4a = {'color':'brown', 'linewidth':1.5, 'linestyle' : '-.'}
style_5 = {'color':'green', 'linewidth':2.5, 'linestyle' : '-.'}
style_6 = {'color':'cyan', 'linewidth':2., 'linestyle' : '--'}
style_7 = {'color':'steelblue', 'linewidth':3., 'linestyle' : '-.'}
style_8 = {'color':'violet', 'linewidth':2., 'linestyle' : '-'}
style_grey = {'color' : 'grey', 'linewidth':4, 'linestyle' : '-'}

## different profile directions
style_Dx = {'fillstyle' : 'left', 'color' : 'red', 'markerfacecolor' : 'red'}
style_Dy = {'fillstyle' : 'bottom', 'color' : 'green', 
            'markerfacecolor' : 'green' }
style_Dz = {'fillstyle' : 'right', 'color' : 'blue', 
            'markerfacecolor' : 'blue'}
style_Du = {'fillstyle' : 'none', 'color' : 'k', 
            'markerfacecolor' : 'white'}
style_Dx_line = {'linestyle' : '--', 'color' : 'red'}
style_Dy_line = {'linestyle' : '-.', 'color' : 'green'}
style_Dz_line = {'linestyle' : ':', 'color' : 'blue'}
style_Du_line = {'linestyle' : '-', 'color' : 'black'}


def get_iorient(orient):
    """Converts x, y, z, u and a, b, c, u to 0, 1, 2, 3. 
    This is a helper function for bound methods in class diffusivitiy 
    and for determining thickness from raypath for wholeblocks"""
    if orient == 'x' or orient == 'a':
        iorient = 0
    elif orient == 'y' or orient == 'b':
        iorient = 1
    elif orient == 'z' or orient == 'c':
        iorient = 2
    elif orient == 'u' or orient == None:
        iorient = 3
    else:
        iorient = orient
    return iorient

        
def ylim_picker(spectrum, wn_xlim_left=4000, wn_xlim_right=3000, pad_top=0.1, 
                pad_bot=0., raw_data=False):
    """
    Takes a Spectrum object and returns reasonable min and max values for 
    y-axis of plots based on the absorbance values for the specified wavenumber
    range and padded top and bottom with pad variable
    """
    try:
        if spectrum.thickness_microns is None:
            absorbance = spectrum.abs_raw
        else:
            spectrum.start_at_zero(wn_xlim_left=wn_xlim_left,
                               wn_xlim_right=wn_xlim_right)
            absorbance = spectrum.abs_full_cm
    except AttributeError:
        absorbance = spectrum.abs_raw
        
    idx_lo = (np.abs(spectrum.wn_full-wn_xlim_right)).argmin()
    idx_hi = (np.abs(spectrum.wn_full-wn_xlim_left)).argmin()
    
    y = absorbance[idx_lo:idx_hi]

    bottom = min(y) 
    top = max(y)
    ylow = bottom - pad_bot
    yhigh = top + pad_top

    return ylow, yhigh
          

def make_line_style(direction, style_marker):
    """Take direction and marker style and return line style dictionary
    that reflects the direction (x, y, z, or u for unoriented) with the 
    color of the base style"""
    if direction == 'x':
        d = style_Dx_line
    if direction == 'y':
        d = style_Dy_line
    if direction == 'z':
        d = style_Dz_line
    if direction == 'u':
        d = style_Du_line
    d.update({'linewidth' : 2})
    return d


def plot_spectrum_outline(size_inches=(6, 6), shrinker=0.15,
                          figaxis=None, wn_xlim_left=4000., 
                          wn_xlim_right=3000., pad_top=0.1, 
                          pad_bot=0., raw_data=False):
    """
    Makes a standard figure outline for plotting FTIR spectra.
    Returns the figure and axis handles.
    """
    if figaxis is None:
        f, ax = plt.subplots(figsize=size_inches)
    else:
        ax = figaxis
    ax.set_xlabel('Wavenumber (cm$^{-1})$')
    ax.set_ylabel('Absorbance (cm$^{-1})$')
    ax.set_xlim(wn_xlim_left, wn_xlim_right)
    ax.grid()    
    box = ax.get_position()
    ax.set_position([box.x0 + box.width*shrinker, 
                     box.y0 + box.height*shrinker, 
                     box.width*(1.0-shrinker), 
                     box.height*(1.0-shrinker)])

    plt.setp(ax.get_xticklabels(), rotation=45)
    
    if figaxis is None:
        return f, ax
    

def plot_area_profile_outline(centered=True, peakwn=None,
                              set_size=(6.5, 4), ytop=1.2, 
                              wholeblock=False, heights_instead=False,
                              show_water_ppm=True):
    """
    Set up area profile outline and style defaults. 
    Default is for 0 to be the middle of the profile (centered=True).
    """
    fig = plt.figure(figsize=set_size)
    ax = SubplotHost(fig, 1,1,1)
    fig.add_subplot(ax)

    ax_ppm = ax.twinx()
    ax_ppm.axis["top"].major_ticklabels.set_visible(False)
    
    if show_water_ppm is True:
        pass
    else:
        ax_ppm.axis["right"].major_ticklabels.set_visible(False)    
    
    ax.set_xlabel('Position ($\mu$m)')
    
    # Set y-label
    if wholeblock is True:
        if heights_instead is False:
            ax.set_ylabel('Area/Area$_0$')
        else:
            ax.set_ylabel('Height/Height$_0$')            
    else:
        if heights_instead is False:
            ax.set_ylabel('Area (cm$^{-2}$)')
        else:
            ax.set_ylabel('Height (cm$^{-1}$)')

    ax.set_ylim(0, ytop)

    ax.grid()
    return fig, ax, ax_ppm


def plot_3panels_outline(style=None, ytop=1.2, figsize=(6.5, 2.5),
                         shrinker=0.1, heights_instead=False,
                         wholeblock=True, unit='microns'):
    """Outline setup for 3 subplots for 3D profiles"""
    if style is None:
        style = style_lightgreen

    fig, axis3 = plt.subplots(nrows=1, ncols=3)
    fig.set_size_inches(figsize)

    for k in range(3):
        axis3[k].set_ylim(0, ytop)
        box = axis3[k].get_position()
        plt.setp(axis3[k].xaxis.get_majorticklabels(), rotation=45)
        axis3[k].set_position([box.x0 + box.width*shrinker, 
                               box.y0 + box.height*shrinker, 
                               box.width*(1.0-shrinker), 
                               box.height*(1.0-shrinker)])
                               
    if wholeblock is True:
        if heights_instead is False:
            axis3[0].set_ylabel('Area/Area$_0$')
        else:
            axis3[0].set_ylabel('Height/Height$_0$')
            
    else:
        if heights_instead is False:
            axis3[0].set_ylabel('Area (cm$^{-2}$)')
        else:
            axis3[0].set_ylabel('Height (cm$^{-1}$)')
        
    axis3[0].set_xlabel('|| x')
    if unit == 'microns':
        axis3[1].set_xlabel('position ($\mu$m) || y')
    elif unit == 'mm':
        axis3[1].set_xlabel('position (mm) || y')
    else:
        print('unit must = microns or mm')
    axis3[2].set_xlabel('|| z')
    plt.setp(axis3[1].get_yticklabels(), visible=False)
    plt.setp(axis3[2].get_yticklabels(), visible=False)
    return fig, axis3
    

def plot_3panels(positions_microns, area_profiles, lengths=None,
                 styles3=[None, None, None], ytop=1.2, figaxis3=None, 
                 show_line_at_1=True, init=1., 
                 centered=True, unit='microns',
                 percent_error=3., xerror=50., yerror=None,
                 heights_instead=False, wholeblock=True,
                 use_errorbar=False, scale=1.):
    """
    Make 3 subplots for 3D and 3DWB profiles. The position and area profiles
    are passed in lists of three lists for a, b, and c.
    """
    if figaxis3 is None:
        fig, axis3 = plot_3panels_outline(ytop=ytop, wholeblock=wholeblock,
                                          heights_instead=heights_instead,
                                          unit=unit)
    else:
        axis3 = figaxis3

    if lengths is None:
        lengths = np.ones(3)
        for k in range(3):
            lengths[k] = max(positions_microns[k] - min(positions_microns[k]))

    for k in range(3): 
        x = positions_microns[k]
        if unit == 'mm':
            x = np.array(x) / 1000.
        y = np.array(area_profiles[k])

        if len(x) != len(y):
            print('Problem in plot_3panels')
            print('len(x):', len(x))
            print('len(y):', len(y))

        a = lengths[k] / 2.
        pos = x 
        
        current_length = axis3[k].get_xlim()[1]
        if centered is True:
            if current_length < a:
                axis3[k].set_xlim(-a, a)
        else:
            if current_length < lengths[k]:
                axis3[k].set_xlim(0., lengths[k])            

        if show_line_at_1 is True:
            axis3[k].plot([-a, lengths[k]], [init, init], '--k')
            
        if styles3[k] is None:
            styles3[k] = style_lightgreen
                   
        if np.isnan(y).any():
            axis3[k].text(0, axis3[k].get_ylim()[1]/2., 
                         'nan values!\n\nProbably the\ninitial area was 0',
                         horizontalalignment='center', backgroundcolor='w',
                         verticalalignment='center')
                         
        elif np.isinf(y).any():
            infstring = ''.join(('inf values!\n\nProbably the',
                                '\ninitial area was 0\nand a peak grew'))
            axis3[k].text(0, axis3[k].get_ylim()[1]/2., infstring,
                         horizontalalignment='center', backgroundcolor='w',
                         verticalalignment='center')

        else:
            if use_errorbar is True:
                if yerror is None:
                    yerrorplot = np.array(y*scale) * percent_error/100.
                else:
                    yerrorplot = np.ones_like(pos) * yerror
                axis3[k].errorbar(pos, y*scale, 
                                  xerr=xerror, yerr=yerrorplot, **styles3[k])
            else:
                axis3[k].plot(pos, y*scale, **styles3[k])

    if figaxis3 is None:
        return fig, axis3   

        


# -*- coding: utf-8 -*-
"""
Created on Mon Mar 23 10:49:59 2015

@author: Ferriss

Library of diffusivities, both data and fit lines, and 
some functions to play with them. The data in in a CSV in this folder.

The focus is on hydrogen diffusion in olivine and clinopyroxene.
"""
from __future__ import print_function, division, absolute_import
import numpy as np
import pynams
from pynams import Sample
from pynams.diffusion.diffusivities import Diffusivities
import pandas as pd
import os

# pull data from spreadsheet
try:
    file = os.path.join(pynams.__path__[0], 'diffusion', 
                        'literaturevalues.csv')
except FileNotFoundError:
    print('Warning: Could not locate and import literature data')
    exit()
    
olivine = pd.read_csv(file)
olivine = olivine.dropna(how='all') 
olivine.fillna(0, inplace=True)
olivine["paper"] = olivine["Author"] + ' ' + olivine["Year"].map(str)
olivinegroups = olivine.groupby(['paper', 'mechanism', 
                                 'percentpv', 'orientation'])
    
## Uncomment to see group names
#for x, group in olivinegroups:
#    print(x)
#print()

SanCarlosOlivine = Sample(Fe3=0.003, Fe2=0.174, Mg=1.821, Ti=0., Al=0.)

# Mackwell and Kohlstedt 1990
SanCarlosOlivineKM = Sample(Mg=0.907, Fe=0.090)
MK90 = Diffusivities(description='Mackwell & Kohlstedt 1990',
                     sample=SanCarlosOlivineKM)
MK90.fill_in_data(df=olivinegroups, mech='bulk', percentpv=0)
MK90.solve_Ea_D0(printout=False)
MK90.activation_energy_kJmol[2] = MK90.activation_energy_kJmol[0] # from paper
MK90.D0_m2s[2] = 5E-6 # from paper

# Kohlstedt and Mackwell 1998
KM98_fast = Diffusivities(description='Kohlstedt & Mackwell 1998',
                          sample=SanCarlosOlivineKM)
KM98_fast.fill_in_data(df=olivinegroups, mech='bulk', percentpv=0)
KM98_fast.solve_Ea_D0(printout=False) # estimate D0
KM98_fast.activation_energy_kJmol = [145., 180., 110., 0] # from paper

KM98_slow = Diffusivities(description='Kohlstedt & Mackwell 1998',
                          sample=SanCarlosOlivineKM)
KM98_slow.fill_in_data(df=olivinegroups, mech='bulk', percentpv=100)
KM98_slow.solve_Ea_D0(printout=False)

# Demouchy and Mackwell 2003
DM03 = Diffusivities(description='Demouchy & Mackwell 2003',
                     sample=Sample(Fe2=0., Fe3=0., Al=0., Mg=2.0, Ti=0.))
DM03.fill_in_data(df=olivinegroups, mech='bulk', percentpv=100)

# Demouchy and Mackwell 2006
SanCarlosOlivineDM = Sample(Mg=0.904, Fe=0.092)
DM06_fast = Diffusivities(description = 'Demouchy & Mackwell 2006', 
                          sample = SanCarlosOlivineDM)
DM06_fast.fill_in_data(df=olivinegroups, mech='bulk', percentpv=0)

DM06_slow = Diffusivities(description = 'Demouchy & Mackwell 2006', 
                          sample = SanCarlosOlivineDM)
DM06_slow.fill_in_data(df=olivinegroups, mech='bulk', percentpv=100)

# pp and pv mechanisms in olivine
pp = Diffusivities(description='KM98 + DM06 "pp"')
pp.log10D = [d98+d06 for d98, d06 in \
             zip(KM98_fast.log10D, DM06_fast.log10D)]
pp.celsius = [d98+d06 for d98, d06 in \
             zip(KM98_fast.celsius, DM06_fast.celsius)]
pp.solve_Ea_D0(printout=False)

pv = Diffusivities(description='KM98 + DM03 + DM06 "pv"')
pv.log10D = [d98 + d03 + d06 for d98, d03, d06 in \
             zip(KM98_slow.log10D, DM03.log10D,DM06_slow.log10D)]
pv.celsius = [d98 + d03 + d06 for d98, d03, d06 in \
             zip(KM98_slow.celsius, DM03.celsius, DM06_slow.celsius)]
pv.solve_Ea_D0(printout=False) # quite different from DM06 results
pv.D0_m2s = [10**-4.5, 10**-4.5, 10**-1.4, 0] #DM06 pg 8
pv.activation_energy_kJmol = [204, 204, 258, 0] #DM06 pg 8


def mix_olivine_mechanisms(percent_slow, celsius):
    """
    Determines diffusivities assuming that some percentage
    of H diffuses by the fast 'proton-polaron' mechanism, and
    some diffuses by the slow 'proton-vacancy' mechanism.
    
    Required input:
        percentage of H that is moving by the slow mechanism
        temperature in Celsius
    
    Output: list of three log10 diffusivities, || a, b, and c in m2/s that 
    represent a linear mixture of the (unlogged) fast and slow mechanism
    diffusivities.
    """
    Ds_log = pv.whatIsD(celsius, printout=False)[0:3]
    Df_log = pp.whatIsD(celsius, printout=False)[0:3]
    Ds = [(10**logD) * percent_slow/100. for logD in Ds_log]
    Df = [(10**logD) * (100.-percent_slow)/100. for logD in Df_log]
    D = np.array(Ds) + np.array(Df)
    D = np.log10(D)
    return list(D)

# Padron-Navarta et al. 2014
# [Si]: 3613 peak in MgO-buffered Fo, 
# [Si]-[Ti]: 3613 peak in Ti-doped sample
# [Ti]: 3525 peak
# [Mg]: 3220 peak
pnav_Mg = Diffusivities(description = 'Padron-Navarta et al. 2014')
pnav_Mg.fill_in_data(df=olivinegroups, mech='[Mg]', percentpv=100)

pnav_Si = Diffusivities(description = 'Padron-Navarta et al. 2014')
pnav_Si.fill_in_data(df=olivinegroups, mech='[Si]', percentpv=100)

pnav_Ti = Diffusivities(description = 'Padron-Navarta et al. 2014')
pnav_Ti.fill_in_data(df=olivinegroups, mech='[Ti]', percentpv=100)

# Demouchy et al. 2016 hydration at 3GPa
D16 = Diffusivities(description='Demouchy et al. 2016',
                    sample=Sample(Fe=0.092, Mg=0.903)) # Ni=0.005, trace Cr

#%% The rest of this has data in an old format that I haven't had a 
### need to switch over into the CSV but I don't want to discard

##Wanamaker_Si = Diffusivities(
##               description = "San Carlos ol. $V''''_{Si}$\nWanamaker 1994",
##               celsius_u= [1100., 1200.], 
##               logDu = [-10.4, -9.99],
##               basestyle = {'color' : 'k', 'marker' : '*', 'alpha' : 0.5,
##                            'linestyle' : 'none', 'markersize' : 10})
##
##Wanamaker_Mg = Diffusivities(
##               description = "San Carlos ol. $V''_{Me}$\nWanamaker 1994",
##               celsius_u = [1100., 1200., 1300.],
#               logDu = [-11.20, -10.8, -10.3],
#               basestyle = {'color' : 'g', 'marker' : '*', 'alpha' : 0.5,
#                            'fillstyle' : 'none',
#                            'linestyle' : 'none', 'markersize' : 10})
#

##%% clinopyroxene
#H_CrDiopside_Ingrin95 = Diffusivities(
#                        description = ''.join(('H in Cr-rich diopside in air',
#                               '\nmainly 3645 cm$^{-1}$\nIngrin et al. 1995')),
#                        celsius_all = np.array([973, 1073, 1173, 1273]) - 273.,
#                        logDx = [np.log10(3E-14), np.log10(9E-14),
#                                 np.log10(5.5E-13), np.log10(1.9E-12)],
#                        logDz = [np.log10(5E-14), np.log10(13E-14), 
#                                np.log10(4.5E-13), np.log10(12E-13)],
#                        basestyle = {'color' : 'green', 'marker' : '^', 
#                                   'markersize' :  12, 
#                                   'linestyle' : 'none', 'alpha' : 1.})
#
#H_cpx_Wade2008 = Diffusivities(
#                 description = 'H in cpx phenocryst\nWade et al. 2008',
#                 celsius_all = [1100], logDu = [-13.],
#                 basestyle = {'color' : 'indigo', 'marker' : '$\clubsuit$', 
#                            'markersize' :  12,
#                            'markeredgewidth' : 0,
#                            'linestyle' : 'none', 'alpha' : 1.})
#
#H_diopside_Woods00 = Diffusivities(
#                     description = 'Jaipur bulk H\nWoods et al. 2000',
#                     logDx = np.log10(np.array([5e-13, 7e-12, 1.1e-11, 1.5e-11, 
#                                          2e-11, 1.8e-11, 3.5e-12, 3.5e-11])),
#                     logDy = np.log10(np.array([1.5e-12, 3e-12, 2.5e-12, 8e-13, 
#                                                2e-12])),
#                     logDz = np.log10(np.array([2e-12, 7e-12, 1.5e-11, 1.5e-11, 
#                                                6e-12, 3e-11])),
#                     celsius_x = [700, 750, 800, 800, 800, 850, 700, 850],
#                     celsius_y = [750, 800, 800, 800, 850],
#                     celsius_z = [700, 750, 800, 800, 800, 850],
#                     basestyle = {'color' : 'k', 'marker' : 's', 
#                                  'markerfacecolor' : 'turquoise',
#                                  'markersize' :  10, 
#                                  'linestyle' : 'none', 'alpha' : 1.,
#                                  'markeredgewidth' : 0.5})
#                            
#H_cpxBasanite_Xia00 = Diffusivities(
#                      description = ''.join(('H in basanite cpx\n3630 + 3500',
#                            ' cm$^{-1}$\nf$_{02}=10^{-14}$ Xia et al. 2000')),
#                      logDz = [np.log10(1.8E-12), np.log10(6.5E-12)],
#                      celsius_z = [850, 950],
#                      basestyle = {'color' : 'k', 'marker' : '>', 
#                                   'markerfacecolor' : 'k',
#                                   'markersize' :  8, 
#                                   'linestyle' : 'none', 'alpha' : 1.,
#                                   'markeredgewidth' : 1})
#
#H_diopside_noFe = Diffusivities(
#                  description = ''.join(('pure synth. diopside in air',
#                                         '\nSundvall et al. 2009')),
#                  logDx = [-12.3, -14.7], 
#                  celsius_x = np.array([1273, 1073]) - 273.,
#                  logDy = [-12.6, -13.9, -15.1],
#                  celsius_y = np.array([1273, 1173, 1073]) - 273.,
#                  basestyle = {'color' : 'b', 'marker' : '>', 
#                               'markerfacecolor' : 'b', 'markersize' :  8, 
#                               'linestyle' : 'none', 'alpha' : 1.,
#                               'markeredgewidth' : 1})
#
#H_diopside_Sundvall = Diffusivities(
#                      description = ''.join(('synth. Fe-bearing diopside\n',
#                                             'in air; Sundvall et al. 2009')),
#                      logDy = [-13.7, -15.3, -15.9],
#                      celsius_y = np.array([1000., 900., 800.]),
#                      basestyle = {'color' : 'y', 'marker' : 'v', 
#                                   'markerfacecolor' : 'y', 'markersize' :  13, 
#                                   'linestyle' : 'none', 'alpha' : 1.,
#                                   'markeredgewidth' : 1})
#
#Fe_diopside = Diffusivities()
#Fe_diopside.description = 'Fe in diopside\nAzough & Freer 2000'
#Fe_diopside.celsius_u = np.array([950., 1100.])
#Fe_diopside.activation_energy_kJmol = 161.5
#Fe_diopside.D0_m2s = 6.22E-15
##Fe_diopside.logDu = Fe_diopside.makelines_Ea_D0(Ea=161.5, log10D0=10.**(6.22E-15), 
##                                celsius_list=Fe_diopside.celsius_u)
#Fe_diopside.basestyle = {'color' : 'k', 'marker' : 'None', 
#                         'linestyle' : '-', 'alpha' : 1.}
#
#
## H in Kunlun diopside
## 1000 C, in whole-block paper, label: K5
##Dx_wholeblock = ufloat(-13.2, 0.2)
##Dy_wholeblock = ufloat(-13.4, 0.2)
##Dz_wholeblock = ufloat(-13.6, 0.3)
##Dy_slice_FTIR = ufloat(-13.1, 0.3)
##Dz_slice_FTIR = ufloat(-13.1, 0.2)
##Dy_slice_SIMS = ufloat(-13.3, 0.4)
##Dz_slice_SIMS = ufloat(-13.2, 0.4)
##Dy_ave = np.mean([Dy_slice_FTIR, Dy_slice_SIMS, Dy_wholeblock])
##Dz_ave = np.mean([Dz_slice_FTIR, Dz_slice_SIMS, Dz_wholeblock])
##
##
##Kunlun_bulkH = Diffusivities()
##Kunlun_bulkH.description = 'Kunlun diopside\nbulk H, QFM'
##Kunlun_bulkH.celsius_all = [1000.]
##Kunlun_bulkH.logDx = [-13.0]
##Kunlun_bulkH.logDy = [-13.5]
##Kunlun_bulkH.logDz = [-13.5]
##Kunlun_bulkH.basestyle = {'color' : 'black', 'marker' : 'D', 
##                          'markersize' :  markersizefloat, 
##                          'linestyle' : 'none', 'alpha' : 0.5,}
##Kunlun_bulkH.logDx_error = [0.1]
##Kunlun_bulkH.logDy_error = [0.1]
##Kunlun_bulkH.logDz_error = [0.1]
##
##Kunlun_peak3617 = Diffusivities()
##Kunlun_peak3617.description = 'Kunlun diopside\nPeak at 3617 cm$^{-1}$'
##Kunlun_peak3617.celsius_all = [1000.]
##Kunlun_peak3617.logDx = [-13.4]
##Kunlun_peak3617.logDy = [-12.3]
##Kunlun_peak3617.logDz = [-13.5]
##Kunlun_peak3617.logDx_error = [0.4]
##Kunlun_peak3617.logDy_error = [0.03]
##Kunlun_peak3617.logDz_error = [0.2]
##Kunlun_peak3617.basestyle = dict(Kunlun_bulkH.basestyle.items())
##Kunlun_peak3617.basestyle['color'] = 'red'
##
##Kunlun_peak3540 = Diffusivities()
##Kunlun_peak3540.description = 'Kunlun diopside\nPeak at 3540 cm$^{-1}$'
##Kunlun_peak3540.celsius_all = [1000.]
##Kunlun_peak3540.logDx = [-12.7]
##Kunlun_peak3540.logDy = [-12.2]
##Kunlun_peak3540.logDz = [-12.7]
##Kunlun_peak3540.logDx_error = [0.3]
##Kunlun_peak3540.logDy_error = [0.1]
##Kunlun_peak3540.logDz_error = [0.3]
##Kunlun_peak3540.basestyle = dict(Kunlun_bulkH.basestyle.items())
##Kunlun_peak3540.basestyle['color'] = 'blue'
#
## Jaipur diopside, Woods et al. 2001
#Jaipur_bulk = Diffusivities()
#Jaipur_bulk.description = 'Jaipur diopside bulk H\n(Woods et al. 2001)'
#Jaipur_bulk.celsius_x = [700, 750, 800, 800, 800, 850, 700, 850]
#Jaipur_bulk.celsius_y = [750, 800, 800, 800, 850]
#Jaipur_bulk.celsius_z = [700, 750, 800, 800, 800, 850]
#Jaipur_bulk.logDx = np.log10([5e-13, 7e-12, 1.1e-11, 1.5e-11, 2e-11, 1.8e-11, 
#                     3.5e-12, 3.5e-11])
#Jaipur_bulk.logDy = np.log10([1.5e-12, 3e-12, 2.5e-12, 8e-13, 2e-12])
#Jaipur_bulk.logDz = np.log10([2e-12, 7e-12, 1.5e-11, 1.5e-11, 6e-12, 3e-11])
#Jaipur_bulk.basestyle = {'color' : 'green', 'marker' : 's',
#                         'markersize' : markersizefloat, 
#                         'linestyle' : 'none'}
#
#O_diopside_self = Diffusivities()
#O_diopside_self.description = 'O self-diffusion in diopside\nRyerson & McKeegan 1994'
#O_diopside_self.logDz = [np.log10(1.96e-21), np.log10(2.04e-21), np.log10(1.48e-21),
#                         np.log10(2.29e-21), np.log10(1.19e-20), np.log10(2.47e-20),
#                         np.log10(3.63e-20), np.log10(1.9e-20), np.log10(3.17e-20),
#                         np.log10(1.75e-20), np.log10(2.17e-20), np.log10(3.74e-20),
#                         np.log10(8.94e-20), np.log10(9.64e-20), np.log10(1.22e-19)]
#O_diopside_self.celsius_z = [1104, 1104, 1104, 1105, 1150, 1200, 1200, 1200, 
#                             1200, 1202, 1202, 1202, 1251, 1251, 1251]
#O_diopside_self.basestyle = {'color' : 'red', 'marker' : '^', 
#                           'linestyle' : 'none',
#                           'fillstyle' : 'right', 'alpha' : 0.5}                             
#
#U_diopside = Diffusivities()
#U_diopside.description = 'U in diopside\nVan Orman et al. 1998'
#U_diopside.celsius_z = [1150, 1150, 1200, 1200, 1200, 1300, 1300]  
#U_diopside.logDz = [np.log10(4.54e-22), np.log10(9.44e-22), np.log10(2.90e-21),
#                    np.log10(3.95e-21), np.log10(2.35e-21), np.log10(2.35e-21),
#                    np.log10(2.18e-20)]
#U_diopside.basestyle = {'color' : 'orange', 'marker' : 'o', 
#                           'linestyle' : 'none', 'markersize' : 7, 
#                           'fillstyle' : 'right', 'alpha' : 0.5}                             
#
#Th_diopside = Diffusivities()
#Th_diopside.description = 'Th in diopside\nVan Orman et al. 1998'
#Th_diopside.celsius_z = [1150, 1150, 1200, 1200, 1200, 1300, 1300]  
#Th_diopside.logDz = [np.log10(1.59e-21), np.log10(1.2e-21), np.log10(5.26e-21),
#                     np.log10(3.80e-21), np.log10(3.22e-21), np.log10(2.12e-20),
#                     np.log10(2.75e-20)]
#Th_diopside.basestyle = {'color' : 'yellow', 'marker' : 'o', 
#                           'linestyle' : 'none', 'markersize' : 7, 
#                           'fillstyle' : 'right', 'alpha' : 0.5}                             
#
#Ce_diopside = Diffusivities()
#Ce_diopside.description = 'Ce in diopside\nVan Orman et al. 2001'
#Ce_diopside.celsius_z = [1300, 1275, 1250, 1250, 1225, 1200, 1200, 1200, 1200, 
#                         1175, 1150]
#Ce_diopside.logDz = [np.log10(31.9e-21), np.log10(25.0e-21), np.log10(11.5e-21),
#                     np.log10(6.83e-21), np.log10(5.83e-21), np.log10(2.53e-21),
#                     np.log10(4.45e-21), np.log10(4.01e-21), np.log10(4.53e-21),
#                     np.log10(0.68e-21), np.log10(0.62e-21)]
#Ce_diopside.basestyle = {'color' : 'lime', 'marker' : 'o', 
#                           'linestyle' : 'none', 'markersize' : 7, 
#                           'fillstyle' : 'right', 'alpha' : 0.5}                             
#
#Al_diopside = Diffusivities()
#Al_diopside.description = 'Al in diopside; Sautter\net al. EPSL 1988'
#Al_diopside.logDu = [-18.495]
##Al_diopside.logDu_error = [0.66]
#Al_diopside.celsius_u = [1180.]
#Al_diopside.basestyle = {'color' : 'g', 'marker' : 'x',
#                         'fillstyle' : 'full'}
#
#CaMg_diopside_2010 = Diffusivities()
#CaMg_diopside_2010.description = 'Ca-Mg interdiffusion in di.\nZhang et al. 2010'
#CaMg_diopside_2010.logDx = [-19.33, -19.37, -19.92, -19.55, -19.99, -19.97, 
#                            -20.39, -20.41, -20.82, -21.18]
#CaMg_diopside_2010.celsius_x = [1150.00, 1150.00, 1100.00, 1100.00, 1050.00, 
#                                1050.00, 1000.00, 1000.00, 950.00, 950.00]
#CaMg_diopside_2010.logDy = [-19.39, -19.38, -20.06, -20.07, -20.95, -21.08, 
#                            -20.92, -21.18, -21.26, -21.36]
#CaMg_diopside_2010.celsius_y = [1150.00, 1150.00, 1050.00, 1050.00, 1000.00, 
#                                1000.00, 1000.00, 1000.00, 950.00, 950.00]
#CaMg_diopside_2010.logDz = [-19.49, -19.44, -20.12, -20.24, -21.36, -21.38]
#CaMg_diopside_2010.celsius_z = [1150.00, 1150.00, 1050.00, 1050.00, 950.00, 950.00]
#                            
#CaMg_diopside_2010.basestyle = {'color' : 'orange', 'marker' : 's', 
#                                'linestyle' : 'none', 'markersize' : 7,
#                                'fillstyle' : 'full', 'alpha' : 0.8}
#
#CaMg_diopside_1983 = Diffusivities()
#CaMg_diopside_1983.description = 'Ca-Mg interdiffusion in cpx\nBrady & McCallister 1983'
#CaMg_diopside_1983.logDu = np.array([np.log10(1.4E-16), np.log10(6.9E-17), np.log10(5.6E-17),
#                                 np.log10(2.0E-17), np.log10(5.7E-18), np.log10(5.6E-16),
#                                 np.log10(2.8E-16), np.log10(1.7E-16), np.log10(6.9E-17),
#                                 np.log10(8.3E-16), np.log10(4.2E-16), np.log10(5.6E-16)]) - 2.
#CaMg_diopside_1983.celsius_all = [1100., 1100., 1100., 1100., 1100., 1150., 1150., 
#                             1150., 1150., 1200., 1200., 1250]
#CaMg_diopside_1983.basestyle = {'color' : 'orange', 'marker' : 's', 
#                           'linestyle' : 'none', 'markersize' : 9,
#                           'fillstyle' : 'none', 'alpha' : 0.5}
#                           
#FeMg_cpx_2013 = Diffusivities()
#FeMg_cpx_2013.description = 'Fe-Mg interdiffusion in cpx\nMueller et al. 2013'
#FeMg_cpx_2013.logDz = [-21, -20.8, -20.46, -18, -21.89, -21.05, -18.64, -19.75, 
#                       -18.3, -17.52, -19.52, -20.4, -20.46, -19.41, -18.7, 
#                       -20.4, -19.7, -18.7, -19.92, -19.52, -19.82, -19.92]
#FeMg_cpx_2013.celsius_z = [850, 900, 950, 1150, 800, 905, 1106, 1006, 1154, 
#                           1200, 1035, 924, 956, 1048, 1102, 945, 999, 1100, 
#                           1000, 1007, 1007, 1007]
#FeMg_cpx_2013.basestyle = {'color' : 'grey', 'marker' : 'h', 
#                           'linestyle' : 'none', 'markersize' : 7,
#                           'fillstyle' : 'right', 'alpha' : 0.8}
#                         
#FeMg_diopside = Diffusivities()
#FeMg_diopside.description = 'Fe-Mg interdiffusion\nDimanov & Wiedenbeck 2006'
#FeMg_diopside.logDz = np.array([-16.6649, -14.9891, -14.5297, -14.6098, -15.0794, 
#                       -15.6405, -15.9297, -15.8744, -16.139, -15.224, 
#                       -16.0353, -14.1226]) - 2. # original reported in log D (cm2/s)
#FeMg_diopside.celsius_z = [1000, 1150, 1100, 1190, 1100, 1100, 1100, 1100, 
#                           1050, 1150, 1100, 1100]
#FeMg_diopside.basestyle = {'color' : 'green', 'marker' : 'o', 
#                           'linestyle' : 'none',
#                           'fillstyle' : 'right', 'alpha' : 0.5}
#
#MnMg_diopside = Diffusivities()
#MnMg_diopside.description = 'Mn-Mg interdiffusion\nDimanov & Wiedenbeck 2006'
#MnMg_diopside.logDz = np.array([-16.6923, -15.0946, -14.8291, -14.8714, -15.223, 
#                       -15.711, -15.9744, -16.0584, -16.2665, -15.8388, 
#                       -16.2668, -14.3497]) - 2. # original reported in log D (cm2/s)
#MnMg_diopside.celsius_z = [1000, 1150, 1100, 1190, 1100, 1100, 1100, 1100, 
#                           1050, 1150, 1100, 1100]
#MnMg_diopside.basestyle = {'color' : 'yellow', 'marker' : 'o', 
#                           'linestyle' : 'none',
#                           'fillstyle' : 'right', 'alpha' : 0.5}
#
#FeMnMg_diopside = Diffusivities()
#FeMnMg_diopside.description = '(Fe,Mn)Mg interdiffusion in di.\nDimanov & Wiedenbeck 2006'
#FeMnMg_diopside.logDz = np.array([-16.6784, -15.0387, -14.6541, -14.7212, -15.1453, 
#                         -15.6743, -15.9515, -15.9567, -16.1981, -15.4307, 
#                         -16.1358, -14.2215]) - 2. # original reported in log D (cm2/s)
#FeMnMg_diopside.celsius_z = [1000, 1150, 1100, 1190, 1100, 1100, 1100, 1100, 
#                           1050, 1150, 1100, 1100]
#FeMnMg_diopside.basestyle = {'color' : 'g', 'marker' : 's', 
#                           'linestyle' : 'none',
#                           'fillstyle' : 'right', 'alpha' : 0.5}
#
#Ti_diopside = Diffusivities()
#Ti_diopside.description = 'Ti in diopside\nCherniak & Liang 2012'
#Ti_diopside.logDz = [-19.89, -19.90, -20.25, -20.43, -20.98, -20.92, -21.41, 
#                     -21.05, -21.43, -21.58, -22.22, -22.27, -22.24, -22.77]
#Ti_diopside.celsius_z = [1250, 1200, 1200, 1151, 1102, 1102, 1090, 1052, 999,
#                         1000, 952, 954, 905, 905]
#Ti_diopside.basestyle = {'color' : 'b', 'marker' : 'o', 
#                           'linestyle' : 'none', 'markersize' : 7, 
#                           'fillstyle' : 'right', 'alpha' : 0.5}
#He_cpx = Diffusivities()
#He_cpx.description = 'He in cpx\nTrull & Kurz 1993'
#He_cpx.logDu = np.array([np.log10(1.26E-10), np.log10(1.34E-10), 
#                                   np.log10(1.32E-10), np.log10(1.17E-10),
#                                   np.log10(7.24E-10), np.log10(9.29E-10), 
#                                   np.log10(8.04E-10), np.log10(6.52E-10),
#                                   np.log10(1.47E-8), np.log10(1.48E-8), 
#                                   np.log10(9.19E-9), np.log10(5.06E-9), 
#                                   np.log10(4.12E-9)]) - 2.
#He_cpx.celsius_u = [965., 965., 965., 965., 1070., 1070., 1070., 1070.,
#                             1170., 1170., 1170., 1170., 1170.]
#He_cpx.basestyle = {'color' : 'lawngreen', 'marker' : '$\spadesuit$', 
#                    'linestyle' : 'none', 
#                    'markersize' : 10, 'fillstyle' : 'full', 'alpha' : 1.}
#
#Li_cpx_interstitial = Diffusivities()
#Li_cpx_interstitial.description = 'Li in cpx as interstitial\nRichter et al. 2014'
#Li_cpx_interstitial.logDu = np.log10(np.array([9.2E-10, 1.6E-9, 1.6E-9, 
#                                     3.3E-8, 3.7E-8, 1.2E-8, 4.6E-10, 6.4E-12]))
#Li_cpx_interstitial.celsius_u = [900.]*len(Li_cpx_interstitial.logDu)
#Li_cpx_interstitial.basestyle = {'color' : 'darkorchid', 'marker' : '+', 
#                                 'linestyle' : 'none', 
#                                 'markersize' : 10, 'alpha' : 1.}
#                             
#Li_cpx_effective = Diffusivities()
#Li_cpx_effective.description = 'effective Li in cpx\nRichter et al. 2014'
#Li_cpx_effective.logDu = np.log10(np.array([1.8E-10, 2.6E-10, 2.5E-9, 2.1E-9]))
#Li_cpx_effective.celsius_u = [900.]*len(Li_cpx_effective.logDu)
#Li_cpx_effective.basestyle = {'color' : 'darkorchid', 'marker' : '+', 
#                                 'linestyle' : 'none', 'mew' : 3,
#                                 'markersize' : 10, 'alpha' : 1.}
#
#
#def fast_vs_slow(celsius, minutes, lengths_microns=[2000., 2000., 2000.], 
#                 sample=None, printout=False):
#    """ Takes: 
#        (1) temperature in celsius (required)
#        (2) time in minutes (required)
#        (3) sample block geometry as EITHER
#            (a) a sample with thickness information in 3 directions
#            (b) keyword lengths_microns (defaults to 2x2x2 mm)
#    Outputs 
#        (1) a plot showing fast and slow mechanism H diffusion progress
#            in olivine for the given temperature, time, and size based on 
#            Kohlstedt and Mackwell, 1998 Arrhenius law
#        (2) if printout=True, the diffusivities
#    """    
#    if sample is not None:
#        a, b, c = sample.thickness_microns[0:3]
#    else:
#        a, b, c = lengths_microns
#
#        
#    if printout is True:
#        print('slower proton-vacancy pv mechanism diffusivities')
#    D_slow = [KM98_slow.whatIsD(celsius, orient='x', printout=printout), 
#              KM98_slow.whatIsD(celsius, orient='y', printout=printout),          
#              KM98_slow.whatIsD(celsius, orient='z', printout=printout)]
#    
#    if printout is True:
#        print
#        print('faster proton-polaron pp mechanism diffusivites')
#    D_fast = [KM98_fast.whatIsD(celsius, orient='x', printout=printout), 
#              KM98_fast.whatIsD(celsius, orient='y', printout=printout),          
#              KM98_fast.whatIsD(celsius, orient='z', printout=printout)]
#    
#    fig, axes, v, x, y = diffusion3Dnpi(lengths_microns=[a, b, c],
#                                        log10Ds_m2s=D_slow, 
#                                        time_seconds=minutes*60., 
#                                        initial=1., final=0.,
#                                        plot3=True, centered=False)
#
#    v2, x2, y2 = diffusion3Dnpi(lengths_microns=[a, b, c], 
#                                styles3=[st.style_1]*3,
#                                log10Ds_m2s=D_fast, figaxis3=axes,
#                                time_seconds=minutes*60., 
#                                initial=1., final=0., plot3=True, 
#                                centered=False)
#                             
#    return fig, axes
#

#
##%%
#def print_info(Diff):
#    """ Takes Diffusivity object and prints out data nicely for copying """
#    orients = ['a', 'b', 'c', 'u']
#    for d3, temp, orient in zip(Diff.logD, Diff.celsius, orients):
#        for d, t in zip(d3, temp):
#            print(orient, t, d)
#            


# -*- coding: utf-8 -*-
"""
Created on Wed Dec 02 09:33:13 2015

@author: Ferriss

Functions to help with performing and interpreting experiments on 
nominally anhydrous minerals. 


"""
from __future__ import print_function, division, absolute_import
import matplotlib.pyplot as plt
import matplotlib.patches as patches
from matplotlib.path import Path
from scipy import constants
from scipy.interpolate import interp1d
import numpy as np

GAS_CONSTANT = constants.physical_constants['molar gas constant'][0] # J/molK
FARADAY_CONSTANT = constants.physical_constants['Faraday constant'][0]

#%% water concentration as a function of water fugacity   
def solubility_of_H_in_olivine(Celsius, water_fugacity_GPa=None, 
                               pressure_GPa=1, author='Mosenfelder',
                               printout=True):
    """
    Returns fully saturated solubility of olivine in H/10^6 Si and ppm H2O.

    Required: temperature in Celsius and either the pressure in GPa (default=1)
    or the water fugacity in GPa. Setting the water fugacity overrides the
    water fugacity calculated based on the pressure and an assumed temperature
    of 1100 C.
        
    Options for volume change with pressure dV are set by kwarg author.
    Use 'Mosenfelder' (default) for dV from Mosenfelder et al. 2006, 
    'Kohlstedt' for Kohlstedt et al. 1996, or 'Zhao' for Zhao et al. 2004
    """
    Kelvin = Celsius + 273.15
    
    if water_fugacity_GPa is None and pressure_GPa is None:
        print('Required: either water_fugacity in GPa or pressure in GPa')
        return
    
    if author == 'Mosenfelder':
        # Mosenfelder et al. 2006
        dV = 10.2e-6 # m^3/mol
    elif author == 'Kohlstedt':
        # Kohlstedt et al. 1996
        dV = 10.0e-6 # m^3/mol
    elif author == 'Zhao':
        # Zhao et al. 2004
        dV = 10.6e-6 # m^3/mol
    else:
        print('author options are Mosenfelder, Kohlstedt, or Zhao')
        print('defaulting to Mosenfelder et al. 2006')
        dV = 10.2e-6 # m^3/mol

    # water fugacity as a function of pressure at constant temp of 1100 C
    # www.esci.umn.edu/people/researchers/withe012/fugacity.htm
    GPa = [0.01, 0.1, 0.5, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
    fugacity = [0.00997, 0.098, 0.6, 2, 12.13, 55.3, 218.66, 784.98, 2625, 
                8305.9, 25092.63, 72904., 204764.6, 558205.715, 1481665.06]

    if pressure_GPa is None:
        f_spline_fit = interp1d(np.log10(fugacity), GPa, kind='cubic')
        pressure_GPa = f_spline_fit(np.log10(water_fugacity_GPa))

    if water_fugacity_GPa is None:
        f_spline_fit_p2f = interp1d(GPa, fugacity, kind='cubic')
        water_fugacity_GPa = f_spline_fit_p2f(pressure_GPa)

    # The exponent part keeps going to 1 because all the numbers are so small.
    pressure_Nm2 = pressure_GPa * 1e9
    exponent_part = np.exp((-1.0*pressure_Nm2*dV) / (GAS_CONSTANT*Kelvin))
    A = 2.45 # H/10^6 Si / GPa
    n = 1
    C = A * (water_fugacity_GPa**n) * exponent_part
    ppm = C * 60
    if printout is True:
        print('Solubility of H in olivine:', ppm, 'ppm H2O')
    return ppm


def convertHunit(conc, from_unit='H/10^6 Si', to_unit='ppm H2O', phase='Fo90',
             printout=True):
    """
    Convert hydrogen concentrations to/from H/10^6 Si and ppm H2O.
    Based on Table 3 of Denis et al. 2013
    """
    if phase == 'Fo90':
        H_to_1_ppm = 16.35
    elif phase == 'opx':
        H_to_1_ppm = 11.49
    elif phase == 'cpx':
        H_to_1_ppm = 11.61
    else:
        print('Valid options for phase are Fo90, opx, and cpx')
        return
      
    if from_unit == 'H/10^6 Si':
        if to_unit == 'ppm H2O':
            new_conc = conc / H_to_1_ppm
        elif to_unit == 'per m3':
            new_conc = conc * (1.0/308.67) * (1e30)
        else:
            print('only going to units "ppm H2O" and "per m3"')
            return
        
    elif from_unit == 'ppm H2O':
        if to_unit == 'H/10^6 Si':
            new_conc = conc * H_to_1_ppm
        elif to_unit == 'per m3':
            new_conc = (conc * H_to_1_ppm) * (1.0/308.67) * (1e30)
        else:
            print('only going to "H/10^6 Si" or "per m3"')
            return
            
    elif from_unit == 'per m3':
        if to_unit == 'H/10^6 Si':
            new_conc = conc / ((1.0/308.67) * (1e30))
        elif to_unit == 'ppm H2O':
            new_conc = (conc / ((1.0/308.67) * (1e30))) / H_to_1_ppm
        else:
            print('only going to "H/10^6 Si" or "ppm H2O"')
            return
        
    else:
        print('Only going from H/10^6 Si, ppm H2O, and per m3 for now')
        return
        
    if printout is True:
        output = ' '.join(('{:.2f}'.format(conc), from_unit, '=', 
                           '{:.2f}'.format(new_conc), to_unit, 'for', phase))
        print(output)
    return new_conc


def bubble_tower(panel='middle', minor_setting=40, 
                 major_setting=155., major_gas='CO2',
                 target_log10fO2=-14., minor_gas='CO',
                 furnace_diameter_inches=2.25,
                 max_flow_rate=4.0, figsize=(6,5)):
    """
    For use estimating gas mixing ratios and settings for gas mixing 
    furnace in Dave Walker's lab. 
    
    Required input: 
        panel (left or middle - default),
        minor gas (usually CO) minor_setting (default=40)
        major gas (usually CO2) major_setting (default=155)
        
    Flow rate measurements are included in dictionaries.
    The dictionary keys are the needle valve settings, and
    the values are lists of flow rate measurements in mL / seconds
    """
    fig = plt.figure(figsize=figsize)
    ax = fig.add_subplot(111)
    ax.set_xlabel('Needle valve setting')
    ax.set_ylabel('Flow rate mL/s')
    ax.set_ylim(0, max_flow_rate)

    monoxide = dict()
    dioxide = dict()

    if panel == 'left':
        ax.set_xlim(0, 175)

        if major_gas == 'CO2':
            monoxide[30] = [0.]
            monoxide[40] = [0.5/13.5, 1./29.1]
            monoxide[50] = [1./12.6]
            monoxide[60] = [3./23.0]
            monoxide[135] = [50./29.6]
            monoxide[150] = [50./25.2]
            monoxide[168] = [50./19.7]
                    
            dioxide[100] = [50./78.4]
            dioxide[125] = [50./28.3]
            dioxide[150] = [50./17.9]
            dioxide[170] = [50./14.1]
        elif major_gas == 'CO':
            monoxide[100] = [10./15.2]
            monoxide[145] = [50./27.6]
            monoxide[173] = [50./18.4]

            dioxide[60] = [5./17.2]
            dioxide[70] = [10./21.8]
            dioxide[80] = [20./29.0]
            dioxide[90] = [20./21.3]
            dioxide[100] = [30./23.8]
            dioxide[115] = [30./18.0]
            dioxide[116] = [40./32.1, 30./17.7]
            dioxide[117] = [30./17.1, 30./17.7]
            dioxide[119] = [50./29.4]
            dioxide[168] = [50./14.5]
            
        else:
            print('major_gas either CO2 (default) or CO')
            return

    elif panel == 'middle':
        ax.set_xlim(0, 150)

        if major_gas == 'CO2':        
            monoxide[7] = [1./13.2] # jerky
            monoxide[19] = [2./15.3]        
            monoxide[30] = [5./25.1]
            monoxide[40] = [5./18.8]
            
            dioxide[70] = [20./26.0, 20./25.6]
            dioxide[100] = [40./27.55]
            dioxide[145] = [50./18.4]
            dioxide[150] = [50./14.4]
        else:
            print('Only CO2 as major gas on middle panel')
            return
    else:
        print('Valid entries for "panel" are left and middle')
        return
            
    # Change from list of flow rates to a single mean value
    dioxide.update((x, np.mean(y)) for x, y in dioxide.items())
    monoxide.update((x, np.mean(y)) for x, y in monoxide.items())
    
    p_dioxide = np.polyfit(dioxide.keys(), dioxide.values(), 1)
    p_monoxide = np.polyfit(monoxide.keys(), monoxide.values(), 1)

    # plot flow rate measurements
    ax.plot(dioxide.keys(), dioxide.values(), 'o', markerfacecolor='w',
            markeredgecolor='b', markeredgewidth=2, label='CO$_2$')
    ax.plot(monoxide.keys(), monoxide.values(), '^', markerfacecolor='w',
            markeredgecolor='r', markeredgewidth=2, label='CO')
    x = ax.get_xlim()
    ax.plot(x, np.polyval(p_dioxide, x), '-', color='b')
    ax.plot(x, np.polyval(p_monoxide, x), '-', color='r')
    
    # minimum acceptable flow rate for good mixing 
    # ~ 1/10 cross-sectional area of furance
    radius_cm = (furnace_diameter_inches / 2.) * 2.54
    cross_section_area_cm2 = constants.pi * radius_cm * radius_cm
    minflow = cross_section_area_cm2 / 10.
    ax.plot(x, [minflow, minflow], '-g')
    ax.text(x[0]+5, minflow+0.05, 'minimum flow rate for good mixing')
    ax.legend(loc=2)
    
    if major_gas == 'CO2':
        CO2_setting = major_setting
        CO_setting = minor_setting       
    else:
        CO2_setting = minor_setting
        CO_setting = major_setting
        minor_gas = 'CO2'

    yCO2 = np.polyval(p_dioxide, CO2_setting)
    yCO = np.polyval(p_monoxide, CO_setting)

    if major_gas == 'CO2':
        major_flowrate = yCO2
        minor_flowrate = yCO
    else:
        minor_gas = 'CO2'
        major_flowrate = yCO
        minor_flowrate = yCO2

    # Arrows on data
    ax.text(CO2_setting, yCO2, '$\leftarrow*$', fontsize=18, 
            rotation=90, ha='center', va='bottom')
    ax.text(CO_setting, yCO, '$\leftarrow*$', fontsize=18, 
            rotation=90, ha='center', va='bottom')
    
    ax.text(10., 2.0, ''.join(('major gas ', major_gas, '\nsetting ', 
                               '{:.0f}'.format(major_setting), '\n',
                               '{:.2f}'.format(major_flowrate), ' mL/s')))
    
    ax.text(10., 1.3, ''.join(('minor gas ', minor_gas, '\nsetting ', 
                               '{:.0f}'.format(minor_setting), '\n',
                               '{:.2f}'.format(minor_flowrate), ' mL/s')))

    ax.set_title(' '.join((panel, 'panel')))
    percentCO2 = 100. * yCO2 / (yCO2 + yCO)
    ax.text(10., 0.9, ''.join(('{:.2f}'.format(percentCO2), '% CO$_2$')))


def fO2(celsius, bars=1., buffer_curve='QFM'):
    """ 
    Input:
        Temperature in Celcius
        Pressure in bars (default is 1)
        Buffer curve. Options are QFM (default), NNO, and IW
            QFM = quartz - fayalite - magnetite buffer (default)
            NNO = Ni-NiO
            IW = iron-wustite; Fe-FeO

    Output is log10 oxygen fugacity in bars for buffer_curve

    Regression of data from O’Neill (1987b) from Herd, 2008.
    """
    Kelvin = celsius + 273.15

    if buffer_curve == 'QFM':    
        A = -24935.0
        B = 8.489
        C = 0.0
    elif buffer_curve == 'NNO':
        A = -24525.4
        B = 8.944
        C = 0.0
    elif buffer_curve == 'IW':
        A = -27489.
        B = 6.702
        C = 0.055
    else: 
        print('Only QFM, IW, and NNO supported for now')
        return False
        
    logfO2 = ((A / Kelvin) + B + (C * (bars-1.0) / Kelvin))
    return logfO2
    

def furnace_calibration(reading_celsius):
    """Calibration for furnace 4 in Dave Walker's lab at LDEO
    based on gold calibration"""
    real_temp = reading_celsius - (6. / (reading_celsius/1064.))
    print('{:.1f}'.format(real_temp), 'degrees C')
    return real_temp


def log10fO2_from_V(volts, celsius, buffermix='CO-CO2'):
    """
    Input:
        Volts reading from pO2 meter 
        Temperature in Celsius 
        
        
    Returns log10 oxygen fugacity in bars 
    
    Uses the Nernst equation:
    E(Volts) = -(RT/zF)lnQ 
    and for now assumes a mixture of CO and CO2, so Q ~= fO2 ^(1/2) and 
    number of electrons z = 2
    """
    Kelvin = celsius + 273.15
    z = 2.
    exponent_in_Q = -0.5
    
    my_constant = z * FARADAY_CONSTANT / (exponent_in_Q * GAS_CONSTANT * 2.303) 
    logfO2 = -1. * my_constant * volts / (Kelvin)
    
    print('{:.1f}'.format(logfO2), 'log10 oxygen fugacity')
    return logfO2

    
def V_from_log10fO2(log10fO2, celsius, buffermix='CO-CO2'):
    """
    Reverse of log10fO2_from_V
    
    Input:
        Target log10fO2 for furnace 
        Temperature in Celsius 
        
        
    Returns target V reading on pO2 sensor using the Nernst equation:
    E(Volts) = -(RT/zF)lnQ 
    and for now assuming a mixture of CO and CO2, so Q ~= fO2 ^(1/2) and 
    number of electrons z = 2
    """
    Kelvin = celsius + 273.15
    z = 2.
    exponent_in_Q = -0.5    
    my_constant = z * FARADAY_CONSTANT / (exponent_in_Q * GAS_CONSTANT * 2.303) 
    Volts = -1. * log10fO2 * Kelvin / my_constant
    print('\n', '{:.3f}'.format(Volts), 'target Volts on pO2 meter\n')

    
def make_capsule_shape(x, y, height, outerD, innerD, shape='regular'):
    """
    Makes and returns path representing the capsule shape for use in 
    pressure vessel schematics as in pressure_design()
    """
    thick = (outerD - innerD) / 2.
    if shape == 'regular':
        verts = [(x + thick*2 + innerD, y),
                 (x, y),
                 (x, y + height),
                 (x + thick, y + height),
                 (x + thick, y + innerD/2.),
                 (x + thick + innerD/2., y + thick),
                 (x + thick + innerD, y + innerD/2.),
                 (x + thick + innerD, y + height),
                 (x + thick*2 + innerD, y + height),
                 (0., 0.)]
        codes = [Path.MOVETO] + ([Path.LINETO] * 8) + [Path.CLOSEPOLY]
    elif shape == 'suaged':
        th_flap = thick/2.
        verts = [(x + thick*2 + innerD, y),
                 (x, y),
                 (x, y + height + th_flap),
                 (x + thick + innerD/2., y + height + th_flap),
                 (x + thick + innerD/2., y + height),
                 (x + thick, y + height),
#                 (x + thick, y + thick),
                 (x + thick, y + innerD/2.),
                 (x + thick + innerD/2., y + thick),
                 (x + thick + innerD, y + innerD/2.), 
#                 (x + thick + innerD, y + thick),
                 (x + thick + innerD, y + height),
                 (x + thick + innerD/2., y + height),
                 (x + thick + innerD/2., y + height + th_flap),
                 (x + thick*2 + innerD, y + height + th_flap),
                 (x + thick*2 + innerD, y + height - th_flap),
                 (0., 0.)]
        codes = [Path.MOVETO] + ([Path.LINETO] * (len(verts)-2)) + [Path.CLOSEPOLY]
    path = Path(verts, codes)
    return path

style_pressure_medium = {'hatch' : '..', 'facecolor' : 'lightgrey'}
style_graphite = {'hatch' : '/', 'facecolor' : 'dimgrey'}
style_MgO = {'hatch' : None, 'facecolor' : 'white', 'edgecolor' : 'k'}
style_pyrophyllite = {'hatch' : 'xx', 'facecolor' : 'hotpink'}
style_capsule = {'facecolor' : 'orange', 'edgecolor' : 'k'}
style_buffer = {'facecolor' : 'w', 'hatch' : 'xxxxx', 'edgecolor' : 'k'}

       
def pressure_design(capsule_material = 'copper',
                    pressure_medium_material='BaCO$_3$',
                    sleeve_material='pyrophyllite',
                    buffer_material='H$_2$O, Ni,\nNiO, SC ol.,\nSC enst.',
                    lid_shape = 'bevel', # or flat or suaged
                    h_graphite_button=1.5, 
                    h_pressure_medium=33.35,
                    h_graphite_cylinder=33.35,
                    h_sleeve=11.5,
                    h_sleeve_bottom=1.,
                    h_capsule = 9.7,
                    h_lid=1., # total 
                    h_MgO_base=10.6,
                    h_MgO_wafer=1.5,
                    h_MgO_top=10.6,
                    od_pressure_medium=18.9,
                    od_graphite_cylinder=11.5,
                    id_graphite_cylinder=10.0,
                    id_sleeve=8.7,
                    id_capsule=6.7,
                    legend_on=True, 
                    figsize=(3., 3),):
    """Creates and returns figure and axis
    showing experimental setup for a high pressure 
    experiment used to hydrate olivine and potentially
    other nominally anhydrous minerals in the green 4-post piston cylinder 
    at Lamont. All dimensions are input in mm. h=height, d=diameter,
    od=outer diameter, id=inner diameter
    """
    # reset all style labels to None for legend 
    style_capsule['color'] = None
    style_buffer['label'] = buffer_material
    for style in [style_graphite, style_MgO, style_capsule]:
        style['label'] = None
    
    d_graphite_button = od_pressure_medium
    od_MgO_base = id_graphite_cylinder
    od_sleeve = id_graphite_cylinder

    fig = plt.figure(figsize=figsize)
    ax = fig.add_subplot(111)
    ax.set_xlim(0, od_pressure_medium)
    ax.set_xlabel('(mm)')
    ax.set_ylabel('(mm)')
    h_guts = h_MgO_base + h_sleeve + h_MgO_wafer + h_MgO_top
    highest_point = max(h_pressure_medium, h_graphite_cylinder, h_guts)
    ax.set_ylim(0., h_graphite_button + highest_point + 2.)
    plt.tick_params(axis='x', top='off')
    ax.spines['top'].set_visible(False)
    ax.spines['left'].set_visible(False)
    ax.spines['right'].set_visible(False)

    th_gc = (od_graphite_cylinder - id_graphite_cylinder) / 2.
    xgc = (od_pressure_medium - od_graphite_cylinder) / 2.

    style_pressure_medium['label'] = pressure_medium_material
    pressure_medium = patches.Rectangle((0., h_graphite_button), # (x,y)
                                        od_pressure_medium, # width
                                        h_pressure_medium, # height
                                        **style_pressure_medium)

    graphite_button = patches.Rectangle((0., 0.), d_graphite_button,
                                        h_graphite_button, **style_graphite)

    style_graphite['label'] = 'graphite'
    graphite_cylinder = patches.Rectangle((xgc, h_graphite_button), 
                             od_graphite_cylinder, h_graphite_cylinder, 
                             **style_graphite)

    the_guts = patches.Rectangle((xgc + th_gc, h_graphite_button), 
                                 id_graphite_cylinder,
                                 h_graphite_cylinder, facecolor='w')
                             
    MgO_base = patches.Rectangle((xgc+th_gc, h_graphite_button),
                                 od_MgO_base, h_MgO_base, **style_MgO)
   
    # sleeve around capsule
    sleeve_path = make_capsule_shape(x=xgc + th_gc, 
                                     y=h_graphite_button + h_MgO_base,
                                     height=h_sleeve, 
                                     outerD=od_sleeve, 
                                     innerD=id_sleeve)
    if sleeve_material == 'MgO':
        style_sleeve = style_MgO.copy()
    elif sleeve_material == 'pyrophyllite':
        style_sleeve = style_pyrophyllite.copy()
        style_sleeve['label'] = 'pyrophyllite'
    else: 
        print('unknown sleeve material. Assuming pyrophyllite')
        style_sleeve = style_pyrophyllite.copy()
        style_sleeve['label'] = 'pyrophyllite'

    sleeve = patches.PathPatch(sleeve_path, **style_sleeve)

    # capsule
    th_sleeve = (od_sleeve - id_sleeve) / 2.
    if (lid_shape == 'bevel') or (lid_shape == 'flat'):
        capsule_path = make_capsule_shape(x=xgc + th_gc + th_sleeve, 
                       y=h_graphite_button + h_MgO_base + h_sleeve_bottom,
                       height=h_capsule, outerD=id_sleeve, innerD=id_capsule)
    elif lid_shape == 'suaged':
        capsule_path = make_capsule_shape(x=xgc + th_gc + th_sleeve, 
                       y=h_graphite_button + h_MgO_base + h_sleeve_bottom,
                       height=h_capsule, outerD=id_sleeve, innerD=id_capsule,
                       shape='suaged')  
    else:
        print('valid entries for lid_shape are flat, bevel, and suaged')
        capsule_path = make_capsule_shape(x=xgc + th_gc + th_sleeve, 
                       y=h_graphite_button + h_MgO_base + h_sleeve_bottom,
                       height=h_capsule, outerD=id_sleeve, innerD=id_capsule)                          
                       
    if capsule_material == 'copper':
        style_capsule['label'] = 'copper'
        style_capsule['facecolor'] = 'orange'
    elif capsule_material == 'silver':
        style_capsule['label'] = 'silver'
        style_capsule['facecolor'] = 'silver'
    elif capsule_material == 'gold':
        style_capsule['label'] = 'gold'
        style_capsule['facecolor'] = 'gold'
    elif capsule_material == 'platinum':
        style_capsule['label'] = 'platinum'
        style_capsule['facecolor'] = 'lightblue'
    elif capsule_material == 'nickel':
        style_capsule['label'] = 'nickel'
        style_capsule['facecolor'] = 'lightsage'
    else:
        print('unknown capsule material')
        style_capsule['label'] = 'capsule'        
    capsule = patches.PathPatch(capsule_path, **style_capsule)

    # MgO on top
    MgO_wafer = patches.Rectangle((xgc + th_gc, 
                                   h_graphite_button + h_MgO_base + h_sleeve),
                                   od_MgO_base, h_MgO_wafer, **style_MgO)

    style_MgO['label'] = 'MgO'
    MgO_top = patches.Rectangle((xgc + th_gc, 
                    h_graphite_button + h_MgO_base + h_sleeve + h_MgO_wafer),
                    od_MgO_base, h_MgO_top, **style_MgO)

    thermocouple = patches.Rectangle((od_pressure_medium/2.-0.5, 
                    h_graphite_button + h_MgO_base + h_sleeve + h_MgO_wafer),
                    1., h_MgO_top, facecolor='w')
    ax.plot([od_pressure_medium/2-0.15, od_pressure_medium/2.-0.15],
            [h_graphite_button + h_MgO_base + h_sleeve + h_MgO_wafer + h_MgO_top + 2.,
             h_graphite_button + h_MgO_base + h_sleeve + h_MgO_wafer], 
             color='r', linewidth=1)
    ax.plot([od_pressure_medium/2+0.15, od_pressure_medium/2.+0.15],
            [h_graphite_button + h_MgO_base + h_sleeve + h_MgO_wafer + h_MgO_top + 2.,
             h_graphite_button + h_MgO_base + h_sleeve + h_MgO_wafer], 
             color='b', linewidth=1)

    # buffer
    th_capsule = (id_sleeve - id_capsule) / 2.
    buffer_inside = patches.Rectangle((xgc + th_gc + th_sleeve + th_capsule,
                    h_graphite_button + h_MgO_base + h_sleeve_bottom + th_capsule),
                    id_capsule, h_capsule - th_capsule, **style_buffer)

    # capsule lid
    del style_capsule['label'] # so it doesn't appear twice in the legend        
    if lid_shape == 'flat':
        lid = patches.Rectangle((xgc + th_gc + th_sleeve, 
                    h_graphite_button + h_MgO_base + h_sleeve_bottom + h_capsule),
                    id_sleeve, h_lid, **style_capsule)
    elif lid_shape == 'bevel':
        x = xgc + th_gc + th_sleeve
        y = h_graphite_button + h_MgO_base + h_sleeve_bottom + h_capsule
        th_lid = h_lid / 2.
        th_capsule = (id_sleeve - id_capsule) / 2.
        lid_verts = [(x + th_capsule, y), 
                     (x, y),
                     (x, y + th_lid), 
                     (x + id_sleeve, y + th_lid), 
                     (x + id_sleeve, y), 
                     (x + id_sleeve - th_capsule, y), 
                     (x + id_sleeve - th_capsule, y - th_lid), 
                     (x + th_capsule, y - th_lid), 
                     (0., 0.)]
        lid_codes = [Path.MOVETO] + ([Path.LINETO] * 7) + [Path.CLOSEPOLY]
        lid_path = Path(lid_verts, lid_codes)
        lid = patches.PathPatch(lid_path, **style_capsule)
    elif lid_shape == 'suaged':
        th_flap = th_capsule / 2.
        x = xgc + th_gc + th_sleeve + th_flap
        ystart = h_graphite_button + h_MgO_base + h_sleeve_bottom
        y = ystart + h_capsule - th_flap*2
        th_lid = h_lid / 2.
        th_capsule = (id_sleeve - id_capsule) / 2.
        lid_verts = [(x + th_capsule - th_flap, y), 
                     (x, y),
                     (x, y + th_lid), 
                     (x + id_sleeve - th_capsule, y + th_lid), 
                     (x + id_sleeve - th_capsule, y), 
                     (x + id_sleeve - th_capsule - th_flap, y), 
                     (x + id_sleeve - th_capsule - th_flap, y - th_lid), 
                     (x + th_capsule - th_flap, y - th_lid), 
                     (0., 0.)]
        lid_codes = [Path.MOVETO] + ([Path.LINETO] * 7) + [Path.CLOSEPOLY]
        lid_path = Path(lid_verts, lid_codes)
        lid = patches.PathPatch(lid_path, **style_capsule)
    else:
        print('valid entries for lid_shape are flat, bevel, and suaged')
        lid = patches.Rectangle((xgc + th_gc + th_sleeve, 
                    h_graphite_button + h_MgO_base + th_sleeve + h_capsule),
                    id_sleeve, h_lid, **style_capsule)

    ax.add_patch(pressure_medium)
    ax.add_patch(graphite_button)
    ax.add_patch(graphite_cylinder)
    ax.add_patch(the_guts)
    ax.add_patch(MgO_base)
    ax.add_patch(sleeve)
    ax.add_patch(buffer_inside)
    ax.add_patch(MgO_wafer)
    ax.add_patch(MgO_top)
    ax.add_patch(thermocouple)
    ax.add_patch(capsule)
    ax.add_patch(lid)
      
    fig.tight_layout()
    if legend_on is True:
        plt.subplots_adjust(right=0.55, left=0.17, bottom=0.15, top=0.9)
        ax.legend(bbox_to_anchor=(2.25, 0.8), frameon=False)
    else:
        plt.subplots_adjust(right=0.9, left=0.17, bottom=0.15, top=0.9)
    return fig, ax
    



"""
Code for transforming FTIR peak areas into water concentrations and 
handling individual peaks. 

The real meat of the code is over in spectra.py and profiles.py, 
but they both use functions stored here.

"""
from __future__ import print_function, division, absolute_import
from uncertainties import ufloat
import numpy as np


def absorption_coefficients(phase, calibration):
    """
    Input the phase ('ol' or olivine' for olivine or 'cpx' or 'clinopyroxene'
    for clinopyroxen) and the calibration reference
    ('Bell' or 'Withers' for olivine, 'Bell' for cpx), 
    and it returns the absorption coefficient.
    This is so you don't have to look it up over and over again.
    
    References:
        Bell et al. 1995 American Mineralogist for cpx
        Bell et al. 2003 JGR for olivine
        Withers et al. 2012 Chemical Geology for olivine
    """
    allowed_phases = ['ol', 'olivine', 'cpx', 'clinopyroxene'] 
    if phase not in allowed_phases:
        print("phases supported so far:", allowed_phases)
        return False
    
    if (calibration == 'Bell') and (phase in ['cpx', 'clinopyroxene']):
        absorption_coeff = 1.0 / ufloat(7.09, 0.32)

    # Bell et al. 2003        
    elif (calibration == 'Bell') and (phase in ['ol', 'olivine']):
        absorption_coeff = ufloat(0.188, 0.012)
        
    # Withers et al. 2012
    elif (calibration == 'Withers') and (phase in ['ol','olivine']):
        absorption_coeff = ufloat(0.119, 0.006)
        
    else:
        print('Calibrations supported so far:')
        print('   Bell et al. 1995 for cpx')
        print('   Bell et al. 2003 for olivine')
        print('   Withers et al. 2012 for olivine')
        return False

    return absorption_coeff        

    
def area2water(area_cm2, phase='cpx', calibration='Bell'):
    """
    Takes the peak area in cm-2, multiplies by the absorption coefficient, 
    and return the water concentration in ppm H2O
    """
    absorption_coeff = absorption_coefficients(phase=phase, 
                                               calibration=calibration)
    w = absorption_coeff * area_cm2
    return w

        
def make_gaussian(pos, h, w, x=np.linspace(3000, 4000, 150)):
    """
    Make and return Gaussian curve over specified range. This is used
    for peak fitting.
    """
    y = h * np.e**(-((x-pos) / (0.6005615*w))**2)
    return y


def make_peakheights(wb, peaks=[3600, 3525, 3356, 3236]):
    """
    Requires:
        1. wb: a wholeblock object that already has baselines.
        2. peaks: a list of peak wavenumber locations in cm-1
        (default peaks=[3600, 3525, 3356, 3236])
    Creates profiles peak positions and peak_heights using that baseline
    """
    for prof in wb.profiles:
        prof.peakpos = peaks
        prof.peak_heights = [[]]*len(peaks)    
        for pidx, peak in enumerate(peaks): 
            prof.peak_heights[pidx] = []
            for spec in prof.spectra:
                idx = np.abs(peak - spec.base_wn).argmin()
                height = spec.abs_nobase_cm[idx]
                prof.peak_heights[pidx].append(height)



# -*- coding: utf-8 -*-
"""
Created on Thu Mar 19 09:24:43 2015

__init__ file for pynams main package

@author: Ferriss
"""
from __future__ import print_function
from .samples import Sample
from .spectra import Spectrum
from .profiles import Profile
from .blocks import Block
from . import styles
from .diffusion import literaturevalues as dlib
from .diffusion.diffusivities import Diffusivities
from . import example_FTIR_spectra
from .experiments import solubility_of_H_in_olivine
from .experiments import convertHunit
from .experiments import fO2
from .experiments import V_from_log10fO2
from .experiments import log10fO2_from_V



"""
Code for processing and plotting *groups* of FTIR spectra.

The central concept is an object class called Profile, which creates groups 
of Spectrum objects, which are defined in a separate module.

@author Elizabeth Ferriss
"""

from __future__ import print_function, division, absolute_import
import numpy as np
from . import styles
from . diffusion import models
from . import pynams
from .spectra import Spectrum
import uncertainties
from uncertainties import ufloat
import gc
import matplotlib.pyplot as plt
import os.path
import matplotlib.gridspec as gridspec
from matplotlib.ticker import MultipleLocator
import string as string
from mpl_toolkits.axes_grid1.parasite_axes import SubplotHost
import json
import lmfit
import copy

class Profile():
    def __init__(self, name=None,
                 fnames=[], 
                 folder='',
                 positions_microns = np.array([]),
                 length_microns=None,
                 thicknesses_microns=None,
                 time_seconds=None, 
                 sample=None, 
                 direction=None, 
                 raypath=None, 
                 initial_profile=None, 
                 base_low_wn=None, 
                 base_high_wn=None,
                 diffusivity_log10m2s=None, 
                 diff_error=None, 
                 peak_diffusivities=[], 
                 peak_diff_error=[], 
                 ):
        """
        Creates a group of FTIR Spectrum objects that can be handled
        and interpreted together.
        
        name is a description automatically used for labeling plots.
        
        fnames must be a *list* of spectra filenames without the .CSV or 
        .txt extension, just like the fname when making a Spectrum.
        
        folder specifies the location of the FTIR files.
        
        positions_microns is a *list* of locations in the same order as the
        fnames
        
        The thickness of the spectra can be specified now with 
        thicknesses_microns, either using a float or integer, or a list 
        of thicknesses, 
        
        Raypath and direction expressed as 'a', 'b', 'c' with thickness/length
        info contained in sample's length_a_microns, length_b_microns, 
        and length_c_microns.
        
        """
        self.name = name
        self.folder = folder
        self.fnames = fnames
        self.positions_microns = positions_microns
        self.sample = sample
        self.length_microns = length_microns
        self.direction = direction
        self.raypath = raypath
        self.initial_profile = initial_profile
        self.time_seconds = time_seconds
        self.diffusivity_log10m2s = diffusivity_log10m2s
        self.diff_error = diff_error
        self.peak_diffusivities = peak_diffusivities
        self.peak_diff_error = peak_diff_error
        
        try:
            if (self.raypath is not None) and (self.raypath == self.direction):
                print("raypath cannot be the same as profile direction")
                return False
        except AttributeError:
            self.raypath = None

        # Create list of spectra in profile.spectra
        # construct each spectrum from fnames
        fspectra_list = []
        for x in self.fnames:
            newspec = Spectrum(fname=x, folder=self.folder)
            newspec.fname = x
            fspectra_list.append(newspec)
        self.spectra = fspectra_list

        # set sample, raypath, low and high baseline wavenumber for all
        for spec in self.spectra:
            spec.sample = self.sample
            spec.raypath = self.raypath

        # set thickness from direct input or sample + raypath
        if thicknesses_microns is not None:
            self.thicknesses_microns = thicknesses_microns
        elif sample is not None and raypath is not None:
            idx = styles.get_iorient(raypath)
            self.thicknesses_microns = np.mean(sample.thicknesses_microns[idx])
        else:
            self.thicknesses_microns = None

        # set thicknesses if thickness information is available
        self.update_spectra_thicknesses_from_profile()
            
        # set or guess profile length
        if self.length_microns is None:
            if (self.sample is not None) and (self.direction is not None):
                self.set_length_from_sampledirection()
            else:
                if self.positions_microns is not None:
                    maxlen = max(list(self.positions_microns))
                    self.length_microns = maxlen + 0.1*maxlen
                
                
    def get_thicknesses_from_SiO(self, show_plots=False, printout=False,
                               accept_thickness=True):
        """
        Individually set thicknesses for all spectra based on the area
        under their Si-O overtone peaks
        """
        for spec in self.spectra:
            spec.get_thickness_from_SiO(show_plot=show_plots,
                                        printout=printout,
                                        accept_thickness=accept_thickness)
        if accept_thickness is True:
            self.update_profile_thicknesses_from_spectra()


    def update_profile_thicknesses_from_spectra(self):
        """
        Gathers thicknesses from spectra into list profile.thicknesses_microns
        """
        thicknesses = []
        for spec in self.spectra:
            try:
                thick = spec.thickness_microns
            except AttributeError:
                print('Spectrum has no thickness. Cannot update profile.')
                return
            thicknesses.append(thick)            
            self.thicknesses_microns = thicknesses        


    def update_spectra_thicknesses_from_profile(self):
        """
        Updates spectra thickness information based on what is in 
        profile.thicknesses_microns
        """
        th = self.thicknesses_microns
        if (isinstance(th, float)) or (isinstance(th, int)):
            for spec in self.spectra:
                spec.thickness_microns = th
        elif isinstance(th, list):
            if len(th) == len(self.fnames):
                for idx, spec in enumerate(self.spectra):
                    spec.thickness_microns = th[idx]
            else:
                print('length of thicknesses_microns not equal length fnames')

            
    def set_length_from_sampledirection(self):
        """
        Set profile.length_microns from profile.direction and 
        profile.sample.thickness_microns
        """ 
        if self.sample is None:
            print('\n', self.name)
            print('Need to specify profile sample\n')
            return False
        else:
            s = self.sample
        if self.direction == 'a':
           self.length_microns = s.thickness_microns[0]
        elif self.direction == 'b':
            self.length_microns = s.thickness_microns[1]
        elif self.direction == 'c':
            self.length_microns = s.thickness_microns[2]
        else:
            print('Set direction of profile to a, b, or c')

        return self.length_microns


    def set_thickness_from_sample(self):
        """
        Set profile.thickness_microns from profile.raypath and
        profile.sample.thickness_microns
        """
        if self.sample is None:
            print('Need to specify profile sample or thickness')
            return False
        else:
            s = self.sample

        if self.raypath == 'a':
           self.thickness_microns = s.thickness_microns[0]
        elif self.raypath == 'b':
            self.thickness_microns = s.thickness_microns[1]
        elif self.raypath == 'c':
            self.thickness_microns = s.thickness_microns[2]
        else:
            print('Need raypath')
            return False
            
        return self.thickness_microns


    def plot_thicknesses(self, axes=None):
        """Plot thickness across profile"""
        if self.length_microns is None:
            self.length_microns = max(self.positions_microns)+1.
        if axes is not None:
            ax = axes
        else:
            fig, ax, ax_right = styles.plot_area_profile_outline(
                                                            centered=False)
        ax.plot(self.positions_microns, self.thickness_microns_list, 'o')
        ax.set_ylabel('thickness ($\mu$m)')
        ax.set_title(self.name)
        ax.set_ylim(min(self.thickness_microns_list)-0.05*min(self.thickness_microns_list), 
                    max(self.thickness_microns_list)+0.05*max(self.thickness_microns_list))
        return fig, ax            


    def average_spectra(self, ignore_idx=[]):
        """
        Creates and returns a spectrum that is the average of all spectra
        in the profile.
        
        Pass a list of the indexes of any spectra that you don't included 
        to ignore_idx 
        """
        spec_list = copy.deepcopy(self.spectra)
        
        # ignore certain data points
        if ignore_idx is None:
            pass
        elif type(ignore_idx) != list:
            print('ignore_idx takes a list')
        elif len(ignore_idx) == 0:
            pass
        else:
             spec_list = [i for j, i in enumerate(spec_list) if j not in ignore_idx]
             
        avespec = Spectrum(folder=None, fname=None)
        avespec.make_average_spectra(spec_list, folder=self.folder)
        
        if self.name is not None:
            avespec.fname = (self.name + '\naveraged across profile')
        else:
            avespec.fname = 'averaged across profile'
            
        avespec.thickness_microns = np.mean(self.thicknesses_microns)
        return avespec


    def plot_spectra(self):
        """
        Plot all spectra across profile with default spectrum.plot_spectrum()
        method. For more detailed inputs or to return figure and axes handles,
        loop through profile.spectra
        """
        for spec in self.spectra:
            spec.plot_spectrum()
            
            
    def plot_showbaselines(self, axes=None, style=styles.style_spectrum, 
                          style_base=styles.style_baseline,
                          label=None, label_baseline=False,
                          offset=0.0):
        """
        Plot all spectra with baselines in profile
        """
        for spec in self.spectra:
                spec.plot_showbaseline(axes=axes, style=style, 
                                       style_base=style_base, label=label,
                                       label_baseline=label_baseline,
                                       offset=offset)
            
            
    def plot_subtractbaselines(self):
        """
        Plot all spectra.plot_subtractbaseline() with default settings
        """
        for spec in self.spectra:
            spec.plot_subtractbaseline()


    def change_baseline(self, highwn=3800, lowwn=3000, shift=None):
        """Change baseline parameters for all spectra, final and initial"""
        for spectrum in self.spectra + self.initial_profile.spectra:
            spectrum.base_high_wn = highwn
            spectrum.base_low_wn = lowwn
            if shift is not None:
                spectrum.base_mid_yshift = shift


    def make_composite_peak(self, peak_idx_list):
        """
        Takes a list of peak indices and makes a new composite
        peak for all spectra in profile        
        """
        for spec in self.spectra:
            spec.make_composite_peak(peak_idx_list)
        self.get_peak_info()


    def make_baselines(self,
                      raw_data=False, 
                      wn_low=3200, 
                      wn_high=3700, 
                      linetype='line', 
                      spline_kind='cubic', 
                      spline_wn_low=3000,
                      spline_wn_high=4000,
                      curvature=None, 
                      force_through_wn=None,
                      polynomial_order=None,
                      show_fit_values=False, 
                      show_plot=False,
                      abs_high=None, 
                      abs_low=None,
                      abs_smear_high=0, 
                      abs_smear_low=0,
                      store_baseline=True
                      ):
        """Make baselines for all final and initial spectra in profile"""
        for spectrum in self.spectra:
            spectrum.base_high_wn = wn_high
            spectrum.base_low_wn = wn_low
            spectrum.make_baseline(raw_data=raw_data, wn_low=wn_low, 
                                   wn_high=wn_high, linetype=linetype, 
                                   spline_kind=spline_kind, 
                                   spline_wn_high=spline_wn_high,
                                   spline_wn_low=spline_wn_low,
                                   curvature=curvature,
                                   force_through_wn=force_through_wn,
                                   polynomial_order=polynomial_order,
                                   show_fit_values=show_fit_values,
                                   show_plot=show_plot, abs_high=abs_high,
                                   abs_low=abs_low, 
                                   abs_smear_high=abs_smear_high,
                                   abs_smear_low=abs_smear_low,
                                   store_baseline=store_baseline)


    def get_baselines(self, folder=None, delim=',', 
                     baseline_ending='-baseline.CSV',
                     print_confirmation=True):
        """
        Get previously saved baselines for all spectra in profile.
        Automatically updates the profile.areas for these baselines.
        """
        for spectrum in self.spectra:
            spectrum.get_baseline(folder=folder, delim=delim, 
                                  baseline_ending=baseline_ending,
                                  print_confirmation=print_confirmation)
        self.make_areas()

                        
    def save_baselines(self, folder=None, delim=',',
                      baseline_ending='-baseline.CSV'):
        """Save all baselines in profile"""
        for spectrum in self.spectra:
            spectrum.save_baseline(folder=folder, delim=delim,
                                   baseline_ending=baseline_ending)
        

    def make_areas(self, peak=None, show_plot=False, 
                       printout_area=False, wn_low=None, wn_high=None):
        """
        Make list of areas under the curve for an FTIR profile.
        Default is bulk area. Set peak=wavenumber for peak-specific profile.
        Change wn_low and wn_high for wavenumber range other than 
        full baseline range, as in spectrum.make_area.
        
        If show_plot and printout_area are set to True, then the plot and
        areas will show up when it calculates the areas under the curve
        for each spectrum.
        """
        areas = []
        if peak is None:
            for spec in self.spectra:
                a = spec.make_area(show_plot=show_plot, 
                                              printout=printout_area,
                                              wn_low=wn_low, wn_high=wn_high)
                areas.append(a)            
            self.areas = np.array(areas)
            
        else:
            peaklist = list(self.spectra[0].peakpos)
            print('peak at', peak)

            if peak in peaklist:
                idx = peaklist.index(peak)
                for x in self.spectra:
                    a = x.peak_areas[idx]
                    areas.append(a)
            else:
                print('No peak at wavenumber', peak)
                print('peak positions:', peaklist)
                return False
        return areas

        
    def make_peakheights(self, peaks=[3600, 3525, 3356, 3236]):
        """
        Requires a list of peak wavenumber locations in cm-1
            (default peaks=[3600, 3525, 3356, 3236])
        Creates or overwrites profiles peak positions and peak_heights using 
        that baseline, not heights from gaussian curves
        """
        self.peakpos = peaks
        self.peak_heights = []
        for peak in (peaks):
            heights = []
            for spec in self.spectra:
                idx = np.abs(peak - spec.base_wn).argmin()
                height_base = spec.base_abs[idx]
                idx = np.abs(peak - spec.wn_full).argmin()
                height_abs = spec.abs_full_cm[idx]                        
                height = height_abs - height_base
                heights.append(height)
            self.peak_heights.append(heights)


    def make_peakfit_like(self, spectrum):
        """
        Exactly like spectrum.make_peakfit_like() but applies the 
        input spectrum's peakfitting information to all spectra in 
        the profile. Any deviations along the profile will have
        to be handfit spectrum by spectrum for now. To do this for
        the first individual spectrum within the profile:
            profile.spectra[0].make_peakfit()
            profile.spectra[0].save_peakfit()
        and then change the index from 0 to whatever spectrum
        you want to look at next
        """
        for spec in self.spectra:
            spec.make_peakfit_like(spectrum)

            
    def save_peakfits(self, folder=None, peak_ending='-peakfit.CSV'):
        """
        Saves all peakfits in the file. Same keywords as 
        spectrum.save_peakfit
        """
        for spec in self.spectra:
            spec.save_peakfit(folder=folder, peak_ending=peak_ending)


    def make_peakfits(self, sensitivity=40, peak_positions=None,
                      peak_heights=None, peak_widths=None,
                      show_plots=False):
        """ 
        Applies Spectrum method make_peakfit() to all spectra in profile
        """
        for spec in self.spectra:
            spec.make_peakfit(sensitivity=sensitivity, 
                              peak_positions=peak_positions,
                              peak_heights=peak_heights, 
                              peak_widths=peak_widths,
                              show_plot=show_plots)

        self.get_peak_info()
        

    def get_peakfits(self, peak_ending='-peakfit.CSV'):
        """
        Get saved fit peak information for all spectra in profile and
        any initial profile that have been saved using the 
        spectrum.save_peakfit or profile.save_peakfits methods.
        
        Change the peak_ending keyword (default='-peakfit.CSV') to 
        specify a particular saved peakfit.
        """
        for spectrum in self.spectra:
            spectrum.get_peakfit(peak_ending=peak_ending)
        try:
            for spectrum in self.initial_profile.spectra:
                spectrum.get_peakfit(peak_ending=peak_ending)
        except AttributeError:
            pass

        if len(self.spectra) > 0:        
            self.get_peak_info()

           
    def get_peak_info(self):
        """
        Pull peak info from individual spectra into a single profile
        attribute
        """
        try:
            peakpos = self.spectra[0].peakpos
        except AttributeError:
            return

        if peakpos is None:
            return False
        hbig = []
        wbig = []
        abig = []
        
        for p in range(len(peakpos)):
            h = []
            w = []
            a = []        
            for spectrum in self.spectra:
                h.append(spectrum.peak_heights[p])
                w.append(spectrum.peak_widths[p])
                try:
                    a.append(spectrum.peak_areas[p])
                except AttributeError:
                    print('Problem with peak areas')
                    a = 0.
            hbig.append(h)
            wbig.append(w)
            abig.append(a)
                
        self.peak_heights = np.array(hbig)
        self.peak_widths = np.array(wbig)
        self.peak_areas = np.array(abig)
        self.peakpos = peakpos
        
        self.peak_wb_areas = np.zeros_like(peakpos)
        self.D_peakarea = np.zeros_like(peakpos)
        self.D_peakarea_error = np.zeros_like(peakpos)
        self.D_peakarea_wb = np.zeros_like(peakpos)
        self.D_peakarea_wb_error = np.zeros_like(peakpos)
        self.peak_maximum_areas = np.zeros_like(peakpos)
        self.peak_maximum_areas_wb = np.ones_like(peakpos)        
        
        self.D_height = np.zeros_like(peakpos)
        self.D_height_error = np.zeros_like(peakpos)
        self.D_height_wb = np.zeros_like(peakpos)
        self.D_height_wb_error = np.zeros_like(peakpos)
        self.peak_maximum_heights = np.zeros_like(peakpos)
        self.peak_maximum_heights_wb = np.ones_like(peakpos)


    def get_area_total(self):
        """Sum up total area of all peaks"""
        total_peak_area = []
        total_height = []
        
        for spec in self.spectra:
            spec.get_peakareas()
            total_peak_area.append(sum(spec.peak_areas))
            total_height.append(sum(spec.peak_heights))
        total_area = np.mean(total_peak_area)
        total_h = np.mean(total_height)
        self.total_peak_area = total_area
        self.total_peak_height = total_h
        return total_area, total_h

            
    def print_peakfits(self):
        """Print out peakfit information for all spectra in profile"""
        print('\n', self.name)

        poscounter = 0
        for spectrum in self.spectra:
            print('\n', spectrum.fname, \
                    self.positions_microns[poscounter], 'microns')
            poscounter += 1
            
            if spectrum.peakpos is None:
                check = spectrum.get_peakfit()
                if check is False:
                    print('trouble with getting peakfit')
            
            if spectrum.peakpos is not None:
                for k in range(len(spectrum.peakpos)):
                    print(spectrum.peakpos[k], spectrum.peak_heights[k], \
                          spectrum.peak_widths[k], spectrum.peak_areas[k])        


    def print_peakfits_ave(self, printout=True):
        """Computes, prints, and returns average values in profile for
        peak areas, peak heights, and the summed average peak areas"""
        # get peak information for all spectra
        for spectrum in self.spectra:
            if spectrum.peak_areas is None:
                self.get_peakfit()

        average_peakareas = np.average(self.peak_areas, axis=1)
        average_peakheights = np.average(self.peak_heights, axis=1)
        total_area = np.sum(average_peakareas)
        
        max_peakareas = np.max(self.peak_areas, axis=1)
        max_peakheights = np.max(self.peak_heights, axis=1)

        if printout is True:
            print('\n', self.name)
            print('peak positions (cm-1)\n', self.spectra[0].peakpos)
            print('average peak areas (cm-2)\n', average_peakareas)
            print('average peak heights (cm-1)\n', average_peakheights)
            print('summed areas (cm-2)', total_area)
            print('max peak areas (cm-2)\n', max_peakareas)
            print('max peak heights (cm-1)\n', max_peakheights)
            
        return average_peakareas, average_peakheights, total_area, \
                max_peakareas, max_peakheights


    def plot_peakfits(self, initial_too=False, legloc=1, ytop=1.2):
        """Show fit peaks for all spectra in profile"""
        if len(self.spectra) < 1:
            self.make_spectra()
            
        for spectrum in self.spectra:
            spectrum.plot_peakfit(legloc=legloc, ytop=ytop)
        
        if initial_too is True and self.initial_profile is not None:
            for spectrum in self.initial_profile.spectra:
                spectrum.plot_peakfit(legloc=legloc)
                

    def make_wholeblock(self, peakfit=False, show_plot=False,
                        heights_or_areas='both'):
        """
        Take initial and final profiles and make self.wb_areas.
        
        If peakfit=True, then make whole-block ratios for heights and
        peak areas (heights_or_areas='both' by default) or just 'heights' or
        'areas'. The results are stored in peak_wb_areas, peak_wb_heights, 
        peak_wb_widths
        """
        if self.initial_profile is None:
            self.initial_profile = self
        init=self.initial_profile

        if peakfit is False:        
            iareas = self.initial_profile.make_areas()
            areas = self.make_areas()
    
            # best-fit line through initial areas        
            if ((len(iareas) == 2) and 
                (init.positions_microns[0] == init.positions_microns[1])):
                p = (0, np.mean(init.areas))
            elif len(iareas) > 1:
                p = np.polyfit(init.positions_microns, iareas, 1)
            else:
                try:
                    p = (0, iareas[0])
                except IndexError:
                    p = (0, 1)
            init_line = np.polyval(p, self.positions_microns)
            
            # whole-block areas relative to that initial line
            area_ratio = self.areas / init_line            
            self.wb_areas = area_ratio
            return

        # Peak-specific whole-block values
        ipos = self.initial_profile.positions_microns
        pos = self.positions_microns       
        
        if heights_or_areas == 'heights' or heights_or_areas == 'both':
            wb_heights = []
            if self.peak_heights is None:
                self.get_peakfit()
                
            try:
                heights = self.peak_heights
            except AttributeError:
                self.get_peakfit()
            try:
                iheights = self.initial_profile.peak_heights
            except AttributeError:
                self.initial_profile.get_peakfit()
                iheights = self.initial_profile.peak_heights
                
            for height, iheight in zip(heights, iheights):
                ph = np.polyfit(ipos, iheight, 1)
                hnormalizeto = np.polyval(ph, pos)
                wb_heights.append(height / hnormalizeto)
            self.peak_wb_heights = wb_heights
                
        if heights_or_areas == 'areas' or heights_or_areas == 'both':
            wb_areas = []
            try:
                areas = self.peak_areas
            except AttributeError:
                print('need profile.peak_areas')
                return
            iareas = self.initial_profile.peak_areas
            for area, iarea in zip(areas, iareas):
                pa = np.polyfit(ipos, iarea, 1)
                anormalizeto = np.polyval(pa, pos)
                wb_areas.append(area / anormalizeto)                
            self.peak_wb_areas = wb_areas            


    def get_peak_wb_areas(self, peak_idx=0, peakwn=None, 
                          heights_instead=False):
        """Returns peak-specific whole-block areas for the profile
        AND peak wavenumber in cm-1 because usually the peak index is easier
        to pass.
        Defaults to the first peak in the peak position list"""
        # Add check that argument is actually a profile
        
        if self.peakpos is None: 
            print('Getting peaks fit in matlab for', self.name)
            self.get_peakfit()

        # Getting peak from wavenumber if index not given
        # Or peak wavenumber from index
        if peak_idx is None:
            if peakwn not in self.peakpos:
                print('There is no peak at', peakwn)
                print('Peaks are at', self.peakpos)
                return
            peak_idx = np.where(self.peakpos==peakwn)[0][0]
            print('peak at', peakwn, 'is index', peak_idx)
        else:
            peakwn = self.peakpos[peak_idx]

        # Make peak-specific whole-block areas
        if heights_instead is True:
            h_or_a = 'heights'
        elif heights_instead is False:
            h_or_a = 'areas'
        else:
            h_or_a = 'both'
        self.make_wholeblock(peakfit=True, show_plot=False,
                             heights_or_areas=h_or_a)
               
        if heights_instead is False:
            returnvals = self.peak_wb_areas[peak_idx]
        else:
            returnvals = self.peak_wb_heights[peak_idx]
    
        return returnvals, peakwn


    def plot_area_profile(self, 
                          axes=None, 
                          show_water_ppm=True,
                          wholeblock=False,
                          peak_idx=None, 
                          centered=False, 
                          normalize_areas=False,
                          normalize_positions=False,
                          heights_instead=False, 
                          bestfitline=False, 
                          style_bestfitline=None,
                          show_FTIR=False, 
                          show_values=False, 
                          peakwn=None, 
                          ytop=None,
                          style=None, 
                          show_initial_areas=False,
                          error_percent=0, 
                          label=None, 
                          initial_style=None,
                          initial_label=None, 
                          phase='olivine',
                          calibration='Bell', 
                          scale_water=3.,
                          shift_water=0., 
                          ignore_idx=[]):
        """
        Plots the area profile. 
        
        If axes is not None, it will plot the area profile to axes. 
        
        If axes is None (default), it will return three things: 
            the figure handle, the right-axes handle showing the absorbance,
            and the left-axes handle showing the rough estimate of the water.
            
        If show_water_ppm is set to False, the left-hand axis will not
        show up, and the third item returned will be None instead of an 
        axes handle.
        
        The water estimate defaults to assuming the phase='olivine' and 
        the calibration='Bell' for the work of Dave Bell, but you can use
        any phase and calibration accepted by the function 
        pynams.absorption_coefficients(). The scale_diffusion (default=3)
        is what the resulting water concentration estimated from the single
        profile is multiplied by to get a rough water estimate. These 
        estimates should be viewed with great skepticism, but it's a place to 
        start. George Rossman's 2006 discussion of how to quantify water in 
        a Reviews in Mineralogy volume is a good place to start for 
        background on these issues. You can shift that scale up or down
        using shift_water (default=0)
        
        Note that the right and left-hand axes are independent of each other,
        and changing the limits on one will not affect the other. To set
        the maximum area (right-hand axis upper limit) and scale both 
        axes simultaneously while making the plot, use the ytop keyword.
        
        Centered=True (False is default) puts 0 in the middle of the x-axis.
        
        If normalize_areas=True (False is default), areas/heights are 
        normalized to the maximum value. If normalize_positions = True
        (default=False), the positions are plotted relative to the length,
        so with resulting x-axis limits from 0 to 1.

        bestfitline=True draws a best-fit line through the data.
        
        Set peak_idx and whole_block for peak-specific and whole-block 
        profiles.
        
        The keyword ignore_idx allows you to pass in a list of indexes
        for points in your profile to exclude from the plot.
        """
        
        if len(self.positions_microns) < 1:
            print('Need positions_microns for profile')
            return

        # Get y data
        if wholeblock is False:
            if peakwn is None and peak_idx is None:
                try:
                    areas = self.areas
                except AttributeError:
                    areas = self.make_areas(peak=None)
            else:
                self.get_peak_info()

                if peak_idx is None:
                    peak_idx = np.where(self.peakpos==peakwn)[0][0]
                    print('peak at', peakwn, 'is index', peak_idx)
                else:
                    peakwn = self.peakpos[peak_idx]

            if heights_instead is True and peak_idx is not None:
                areas = self.peak_heights[peak_idx]
            elif peak_idx is not None:
                areas = self.peak_areas[peak_idx]
        else:
            if self.initial_profile is None:
                print('Need to specify an initial profile')
                return
            if peak_idx is None:
                if self.wb_areas is None:
                    self.make_wholeblock(peakfit=False, bulk=True)
                areas = self.wb_areas
            else:
                if self.peak_wb_areas is None:
                    self.make_wholeblock(peakfit=True, bulk=False)
    
                if heights_instead is False:
                    areas = self.peak_wb_areas[peak_idx]
                else:
                    areas = self.peak_wb_heights[peak_idx]                   
        if areas is False:
            return            
        
        if normalize_areas is True:
            areas_for_water = list(areas)
            areas = np.array(areas) / float(max(areas))
        else:
            areas_for_water = areas
            
        if np.shape(areas) != np.shape(self.positions_microns):
            print('Area and positions lists are not the same size!')
            print('area:', np.shape(self.areas))
            print('positions:', np.shape(self.positions_microns))
            return

        # Set length
        if self.length_microns is None:
            if self.sample is not None:
                leng = self.set_len()
            else:
                longest = max(self.positions_microns)
                leng = longest + 0.1*longest
        else:
            leng = self.length_microns

        # Set up plotting styles
        if style_bestfitline is None:
            style_bestfitline = styles.style_baseline
        if label is None:
            if style is not None:
                style['label'] = self.name
            else:
                label = self.name
        else:
            if style is not None:
                style['label'] = label

          
        # x data
        if normalize_positions is True:
            x = np.array(self.positions_microns) / leng
            if centered is True:
                x = x - 0.5
        else:
            if centered is True:
                x = np.array(self.positions_microns) - leng/2.0
            else:
                x = self.positions_microns            
                
        # ignore certain data points
        if ignore_idx is None:
            pass
        elif type(ignore_idx) != list:
            print('ignore_idx takes a list of integers: indices to ignore')
        elif len(ignore_idx) == 0:
            pass
        else:
             areas = [i for j, i in enumerate(areas) if j not in ignore_idx]
             areas_for_water = [i for j, i in enumerate(areas_for_water) if j not in ignore_idx]
             x = [i for j, i in enumerate(x) if j not in ignore_idx]
             
        # Use new or old figure axes
        if axes is None:
            if ytop is None:
                if peak_idx is None:
                    ytop = max(areas)+0.2*max(areas)
                else:
                    if heights_instead is False:
                        mpeak = max(self.peak_areas[peak_idx])
                    else:
                        mpeak = max(self.peak_heights[peak_idx])
                    ytop = mpeak + 0.2*mpeak
                       
            f, ax, ax_ppm = styles.plot_area_profile_outline(centered, 
                            peakwn, heights_instead=heights_instead, 
                            wholeblock=wholeblock, ytop=ytop,
                            show_water_ppm=show_water_ppm)

            if normalize_areas is True:
                if heights_instead is False:
                    ax.set_ylabel('Area normalized to max. area')
                else:
                    ax.set_ylabel('Height normalized to max. height')
            if normalize_positions is True:
                ax.set_xlabel('Position relative to total length')
                if centered is True:
                    ax.set_xlim(-0.5, 0.5)
                else:
                    ax.set_xlim(0., 1.)
            else:
                if centered is True:
                    ax.set_xlim(-leng/2.0, leng/2.0)
                else:
                    ax.set_xlim(0, leng)
        else:
            ax = axes
            show_water_ppm = False

        # plot
        yerror = np.array(areas)*error_percent/100.       
        if error_percent == 0:
            if style is None:
                ax.plot(x, areas, label=label)
            else:
                ax.plot(x, areas, **style)
        else:
            if style is not None:
                ax.errorbar(x, areas, yerr=yerror, **style)
            else:
                ax.errorbar(x, areas, yerr=yerror, label=label)

            
        # Plot best fit line beneath data points
        if bestfitline is True:
            if ((len(areas) == 2) and 
                (self.positions_microns[0] == self.positions_microns[1])):
                p = (0, np.mean(areas))
            elif len(areas) > 1:
                p = np.polyfit(self.positions_microns, areas, 1)
            else:
                p = (0, areas[0])

            self.bestfitline_areas = p
            x = np.linspace(0, leng, 100)
            y = np.polyval(p, x)
            if centered is True:
                ax.plot(x-leng/2.0, y, **style_bestfitline)
            else:
                ax.plot(x, y, **style_bestfitline)

        # Plot initial profile areas
        if show_initial_areas is True:
            if initial_style is None:
                initial_style = styles.style_initial_point
            if initial_label is None:
                initial_style['label'] = 'initial'
            else:
                initial_style['label'] = label
            self.initial_profile.plot_area_profile(style=initial_style,
                                                   figaxis=ax)            

        # Title
        if peak_idx is None:
            tit = 'Bulk hydrogen'
        else:
            peakwn = self.peakpos[peak_idx]
            tit = ' '.join(('Peak at', str(peakwn) ,'/cm'))
        ax.set_title(tit)

        if show_water_ppm is True:
            abs_coeff = pynams.absorption_coefficients(phase=phase, 
                                                       calibration=calibration)
            if abs_coeff is False:
                show_water_ppm = False

        # The water concentration estimate on the right-hand axis
        if heights_instead is True:
            show_water_ppm = False
        if show_water_ppm is True:
            if shift_water == 0.:
                ax_ppm.set_ylabel(''.join(('ppm H2O in ', phase, ', ', 
                                           calibration, ' calibration *', 
                                           str(scale_water))))
            else:
                ax_ppm.set_ylabel(''.join(('ppm H2O in ', phase, ', ', 
                                           calibration, ' calibration *', 
                                           str(scale_water), ' + ',
                                           str(shift_water))))                
            ori = scale_water
            if normalize_areas is False:
                ppm_limits = np.array(ax.get_ylim()) * abs_coeff.n * ori
            else:
                ppm_limits = np.array([0, max(areas_for_water)*ytop])
                ppm_limits = ppm_limits * abs_coeff.n * ori
            ax_ppm.set_ylim(ppm_limits + shift_water)
        elif axes is None:
            ax_ppm.get_yaxis().set_visible(False)

        if axes is None:
            return f, ax, ax_ppm
        else:
            return


    def diffusion1D(self, log10D_m2s, time_seconds, wholeblock=False,
                    peak_idx=None, centered=True, heights_instead=False, 
                    init=1., fin=0., erf_or_sum='erf', infinity=100, 
                    points=100, symmetric=True, maximum_value=None):
        """
        Requires log10D_m2s and time_seconds
        
        Other keywords like pynams.diffusion.models.diffusion1D
        and profile.plot_diffusion()
        
        Returns x and y for diffusion modeling, without plotting
        """
        try:
            length = self.length_microns
        except AttributeError:
            length = max(self.positions_microns)
        
        if maximum_value is None:
            if peak_idx is None:
                try:
                    maximum_value = max(self.areas)
                except AttributeError:
                    maximum_value = max(self.make_areas())
            else:
                if heights_instead is True:
                    maximum_value = max(self.peak_heights[peak_idx])
                else:
                    maximum_value = max(self.peak_areas[peak_idx])

        fig, ax, x, y = models.diffusion1D(length, log10D_m2s, time_seconds, 
                                           init=init, fin=fin, 
                                           erf_or_sum=erf_or_sum, 
                                           show_plot=False, infinity=infinity, 
                                           points=points, centered=centered,
                                           symmetric=symmetric, 
                                           maximum_value=maximum_value)
        return x, y*maximum_value


    def plot_diffusion(self, log10D_m2s, time_seconds, 
                      axes=None, 
                      show_water_ppm=True,
                      wholeblock=False,
                      peak_idx=None, 
                      centered=True, 
                      heights_instead=False, 
                      style=styles.style_points, 
                      style_diffusion_line=styles.style_blue,
                      phase='olivine',
                      calibration='Bell',
                      scale_water=3,
                      shift_water=0.,
                      init=1., 
                      fin=0.,
                      erf_or_sum='erf', 
                      infinity=100, 
                      points=100, 
                      symmetric=True,
                      maximum_value=None,
                      ytop=None,
                      normalize_areas=False,
                      normalize_positions=False):

        """ 
        Plots area profile and with 1D diffusion profile on top.
    
        Returns figure handle and both left and right axes handles
        like profile.plot_area_profile()
        
        Includes most of the keywords from profile.plot_area_profile
        and diffusion.models.diffusion1D
        """
        fig, ax, ax_water = self.plot_area_profile(axes=axes,
                                       show_water_ppm=show_water_ppm,
                                       wholeblock=wholeblock,
                                       peak_idx=peak_idx,
                                       centered=centered,
                                       heights_instead=heights_instead,
                                       style=style,
                                       phase=phase,
                                       calibration=calibration,
                                       scale_water=scale_water,
                                       shift_water=shift_water,
                                       ytop=ytop,
                                       normalize_areas=normalize_areas,
                                       normalize_positions=normalize_positions)
        try:
            length = self.length_microns
        except AttributeError:
            length = max(self.positions_microns)
        
        if maximum_value is None:
            if peak_idx is None:
                try:
                    maximum_value = max(self.areas)
                except AttributeError:
                    maximum_value = max(self.make_areas())
            else:
                if heights_instead is True:
                    maximum_value = max(self.peak_heights[peak_idx])
                else:
                    maximum_value = max(self.peak_areas[peak_idx])

        models.diffusion1D(length, log10D_m2s, time_seconds, init=init, 
                           fin=fin, erf_or_sum=erf_or_sum, show_plot=True, 
                           style=style_diffusion_line, infinity=infinity, 
                           points=points, centered=centered, axes=ax, 
                           symmetric=symmetric, maximum_value=maximum_value)
       
        return fig, ax, ax_water
    

    def y_data_picker(self, wholeblock, heights_instead, peak_idx=None):
        """
        Pick out and return peak area or height data of interest.
        """
        if wholeblock is True:
            if peak_idx is None:
                if self.wb_areas is None:
                    self.make_wholeblock(peakfit=False, bulk=True)
                y = self.wb_areas
            # Peak-specific 
            else:
                if self.peak_wb_areas is None:
                    self.make_wholeblock(peakfit=True, bulk=False)
                    
                if heights_instead is False:
                    y = self.peak_wb_areas[peak_idx]
                else:
                    y = self.peak_wb_heights[peak_idx]
        else:
            if peak_idx is None:
                try:
                    y = self.areas
                except AttributeError:
                    y = self.make_areas()
            else:
                try:
                    y = self.peak_areas[peak_idx]
                except AttributeError:
                    print('Need peak areas')
                    return False
        return y


    def D_picker(self, wholeblock=True, heights_instead=False, peak_idx=None):
        """Returns attribute name and diffusivity of interest. 
        Consider using get_diffusivity() first"""
        log10D_m2s = None
        
        if peak_idx is not None:
            if heights_instead is True:
                if wholeblock is False and self.D_height[peak_idx] != 0.0:
                    log10D_m2s = self.D_height[peak_idx]
                elif wholeblock is True and self.D_height_wb[peak_idx] != 0.0:
                    log10D_m2s = self.D_height_wb[peak_idx]
            else:
                if wholeblock is False and self.D_peakarea[peak_idx] != 0.0:
                    log10D_m2s = self.D_peakarea[peak_idx]
                elif wholeblock is True and self.D_peakarea_wb[peak_idx] != 0.0:
                    log10D_m2s = self.D_peakarea_wb[peak_idx]
        elif wholeblock is False and self.D_area != 0.0:
            log10D_m2s = self.D_area
        elif self.D_area_wb != 0.0:
            log10D_m2s = self.D_area_wb
            
        if log10D_m2s is None or log10D_m2s == 0.:
            print('\nNeed to set profile bulk or peak diffusivity')
            return None
            
        return log10D_m2s


    def D_saver(self, D, error, wholeblock=True, heights_instead=False, 
                peak_idx=None):
        """
        Saves a diffusivity in the profile attribute of interest.
        Consider using get_diffusivity() first
        """
        if peak_idx is not None:
            if heights_instead is True:
                if wholeblock is False:
                    self.D_height[peak_idx] = D
                    self.D_height_error[peak_idx] = error
                elif wholeblock is True:
                    self.D_height_wb[peak_idx] = D
                    self.D_height_wb_error[peak_idx] = error
            else:
                if wholeblock is False:
                    self.D_peakarea[peak_idx] = D
                    self.D_peakarea_error[peak_idx] = error
                elif wholeblock is True:
                    self.D_peakarea_wb[peak_idx] = D
                    self.D_peakarea_wb_error[peak_idx] = error
        elif wholeblock is False:
            self.D_area = D
            self.D_area_error = error
        else:
            self.D_area_wb = D
            self.D_area_wb_error = error


    def scale_diffusion_picker(self, maximum_value=None, wholeblock=True,
                              heights_instead=False, peak_idx=None):
        """Returns value to scale diffusion to"""
        if maximum_value is not None:
            scale_diffusion = maximum_value
            
        elif wholeblock is True:
            if peak_idx is None:
                scale_diffusion = self.maximum_wb_area
            else:
                if heights_instead is False:
                    scale_diffusion = self.peak_maximum_areas_wb[peak_idx]
                else:
                    scale_diffusion = self.peak_maximum_heights_wb[peak_idx]
        
        else: 
            # bulk water                
            if peak_idx is None:
                if len(self.areas) < 1:
                    self.make_areas()
                scale_diffusion = np.max(self.areas)
        
            # peak-specific heights                
            elif heights_instead is True: 
                if self.peak_maximum_heights[peak_idx] != 0.0:
                    scale_diffusion = self.peak_maximum_heights[peak_idx]
                else:                
                    scale_diffusion = np.max(self.peak_heights[peak_idx])
                    self.peak_maximum_heights[peak_idx] = scale_diffusion
            # peak-specific areas
            else:
                if self.peak_maximum_areas[peak_idx] != 0.0:
                    scale_diffusion = self.peak_maximum_areas[peak_idx]
                else:
                    scale_diffusion = np.max(self.peak_areas[peak_idx])
                    self.peak_maximum_areas[peak_idx] = scale_diffusion
    
        return scale_diffusion


    def diffusion_residuals(self, time_seconds, log10D_m2s, wholeblock=False,
                            heights_instead=False, peak_idx=None,
                            initial_unit_value=1., final_unit_value=0.,
                            maximum_value=None):
        """
        Compares 1D diffusion curve with profile data.
        Returns a vector containing the residuals 
        and the variance, which is sqrt(sum of squares of the residuals)
        """
        if self.positions_microns is None:
            print('Need to set profile positions')
            return

        y = self.y_data_picker(wholeblock, heights_instead, peak_idx)
        scale_diffusion = self.scale_diffusion_picker(maximum_value, 
                                        wholeblock, heights_instead, peak_idx)
        
        x = self.positions_microns
        L = self.length_microns
        t = time_seconds

        # Need to work on this        
        init = scale_diffusion
            
        fin = final_unit_value
        
        params = models.params_setup1D(L, log10D_m2s, t, init, fin, 
                                          False, False, False)

        xdif, model = models.diffusion1D_params(params)
        resid = models.diffusion1D_params(params, x, y)
        RSS = np.sum(resid**2)        
        return resid, RSS
            

    def fitD(self, 
             time_seconds=None, 
             log10Dm2s=None, 
             initial_unit_value=1., 
             final_unit_value=0.,  
             starting_value=None, 
             vary_initial=False,
             vary_final=False,
             vary_time=True, 
             varyD=False, 
             peak_idx=None, 
             wholeblock=False, 
             centered=False,
             show_plot=True, 
             heights_instead=False,
             symmetric=True,
             points=200,
             ignore_idx=[]):
        """
        Fits a 1D diffusion curve to profile data.
        
        Prints and returns: initial, final, log10 diffusivity in m2/s, 
        time in minutes.

        Default fits both diffusivity and initial concentration and
        holds time (default=10 hours) and final unit value constant.
        Set vary_initial=False to hold initial constant. Set 
        vary_time and vary_final to True to fit those. Set varyD = False
        to hold diffusivity constant, and set the diffusivity with the
        keyword log10D_m2s (default is -12).
        
        Default initial is the maximum y-value, but you can change that
        with initial_unit_value or starting_value.
        
        Pass list of indexes of points you don't want included to ignore_idx
        """           
        if time_seconds is None:
            try:
                time_seconds = self.time_seconds
            except AttributeError:
                time_seconds = 36000.
            if self.time_seconds is None:
                time_seconds = 36000.

        if log10Dm2s is None:
            try:
                log10Dm2s = self.D_area
            except AttributeError:
                log10Dm2s = -12.

        if self.length_microns is None:
            print('Need to set profile attribute length_microns')
            return
            
        if self.positions_microns is None:
            print('Need to set profile positions')
            return
        
        x = self.positions_microns.copy()
        y_raw = self.y_data_picker(wholeblock, heights_instead, peak_idx).copy()

        # ignore certain data points
        if ignore_idx is None:
            pass
        elif type(ignore_idx) != list:
            print('ignore_idx takes a list')
        elif len(ignore_idx) == 0:
            pass
        else:
             x = [i for j, i in enumerate(x) if j not in ignore_idx]
             y_raw = [i for j, i in enumerate(y_raw) if j not in ignore_idx]

        scale_diffusion = max(y_raw)
        y = y_raw / scale_diffusion 

             
        if starting_value is None:
            init = initial_unit_value
        else:
            init = starting_value / max(y_raw)

        params = models.params_setup1D(microns=self.length_microns, 
                                       log10D_m2s=log10Dm2s, 
                                       time_seconds=time_seconds, 
                                       init=init,
                                       fin=final_unit_value,
                                       vinit=vary_initial, 
                                       vfin=vary_final,
                                       vTime=vary_time, 
                                       vD=varyD)

        dict_fitting = {'points' : points,
                        'symmetric' : symmetric,
                        'centered' : centered
                        }

        lmfit.minimize(models.diffusion1D_params, params, args=(x, y), 
                       kws=dict_fitting)
        best_D = ufloat(params['log10D_m2s'].value, 
                        params['log10D_m2s'].stderr)
        best_init = ufloat(params['initial_unit_value'].value, 
                         params['initial_unit_value'].stderr)
        best_final = ufloat(params['final_unit_value'].value, 
                         params['final_unit_value'].stderr)
        best_time = ufloat(params['time_seconds'].value,
                           params['time_seconds'].stderr)

        if wholeblock is True:
            if peak_idx is not None:
                if heights_instead is False:
                    self.D_peakarea_wb[peak_idx] = best_D.n
                    self.D_peakarea_wb_error[peak_idx] = best_D.s
                    self.peak_maximum_areas_wb[peak_idx] = best_init.n
                else:
                    self.D_height_wb[peak_idx] = best_D.n
                    self.D_height_wb_error[peak_idx] = best_D.s
                    self.peak_maximum_heights_wb[peak_idx] = best_init.n

            else:
                self.D_area_wb = best_D.n
                self.D_area_wb_error = best_D.s
                self.maximum_wb_area = best_init.n
        
        print('initial:', '{:.2f}'.format(best_init*scale_diffusion))
        print('final:', '{:.2f}'.format(best_final*scale_diffusion))
        print('log10D m2/s:', '{:.2f}'.format(best_D))
        print('minutes:', '{:.2f}'.format(best_time/60))
        if show_plot is True:
            fig, ax, ax2 = self.plot_diffusion(time_seconds=best_time.n,
                                   log10D_m2s=best_D.n,
                                   peak_idx=peak_idx,
                                   wholeblock=wholeblock,
                                   centered=centered,
                                   symmetric=symmetric,
                                   heights_instead=heights_instead, 
                                   maximum_value=best_init.n*scale_diffusion,
                                   fin=best_final.n)
        D = best_D
        init = best_init*scale_diffusion
        fin = best_final*scale_diffusion
        minutes = best_time / 60.
        return init, fin, D, minutes


    def save_diffusivities(self, folder=None, 
                           file_ending='-diffusivities.txt'):
        """Save diffusivities for profile to a file"""
        if folder is None:
            folder = self.folder
            
        if self.short_name is None:
            print('Need profile short_name attribute to label file')
            return
            
        if self.peakpos is None:
            self.get_peakfit()

        # Same format as output by print_diffusivities:
        # Diffuvities from WB areas - area - WB heights - heights 
        # Diffusivity - error - maximum value to scale up to for each

        # Saving diffusivities in first column, errors in 2nd, 
        # Bulk H in first row, then each peak-specific value
        a = []
        a.append([self.D_area_wb, self.D_area_wb_error, self.maximum_wb_area])
        a.append([self.D_area,    self.D_area_error,    self.maximum_area])
        
        for k in range(len(self.peakpos)):
            a.append([self.D_peakarea_wb[k], self.D_peakarea_wb_error[k],
                     self.peak_maximum_areas_wb[k]])
            a.append([self.D_peakarea[k], 
                      self.D_peakarea_error[k], 
                      self.peak_maximum_areas[k]])
            a.append([self.D_height_wb[k], self.D_height_wb_error[k],
                     self.peak_maximum_heights_wb[k]])
            a.append([self.D_height[k],
                     self.D_height_error[k], self.peak_maximum_heights[k]])
        
        workfile = ''.join((folder, self.short_name, file_ending))
        with open(workfile, 'w') as diff_file:
            diff_file.write(json.dumps(a))


    def get_diffusivities(self, folder=None, 
                           file_ending='-diffusivities.txt'):
        """Get saved diffusivities for profile from a file"""
        if folder is None:
            folder = self.folder
        
        # Look into json format        
        if self.short_name is None:
            print('Need profile short_name attribute to label file')
            return
            
        if self.peakpos is None:
            self.get_peakfit()

        workfile = ''.join((folder, self.short_name, file_ending))
        if os.path.isfile(workfile) is False:
            print(' ')
            print(self.fname)      
            print('use save_diffusivities() to make -diffusivities.txt')
            return

        with open(workfile, 'r') as diff_file:
            diffusivities_string = diff_file.read()

        diffusivities = json.loads(diffusivities_string)

        self.D_area_wb = diffusivities[0][0]
        self.D_area_wb_error = diffusivities[0][1]
        self.maximum_wb_area = diffusivities[0][2]
        
        self.D_area = diffusivities[1][0]
        self.D_area_error = diffusivities[1][1]
        self.maximum_area = diffusivities[1][2]
        
        if self.peakpos is None:
            self.get_peakfit()
        if self.peakpos is None:
            print('Having trouble getting peakfit info to grab diffusivities')
            return            
                    
        npeaks = len(self.peakpos)
        for k in range(npeaks):
            self.D_peakarea_wb[k] = diffusivities[2+4*k][0]
            self.D_peakarea_wb_error[k] = diffusivities[2+4*k][1]
            self.peak_maximum_areas_wb[k] = diffusivities[2+4*k][2]
            self.D_peakarea[k] = diffusivities[3+4*k][0]
            self.D_peakarea_error[k] = diffusivities[3+4*k][1]
            self.peak_maximum_areas[k] = diffusivities[3+4*k][2]
            self.D_height_wb[k] = diffusivities[4+4*k][0]
            self.D_height_wb_error[k] = diffusivities[4+4*k][1]
            self.peak_maximum_heights_wb[k] = diffusivities[4+4*k][2]
            self.D_height[k] = diffusivities[5+4*k][0]
            self.D_height_error[k] = diffusivities[5+4*k][1]
            self.peak_maximum_heights[k] = diffusivities[5+4*k][2]
        
        return diffusivities
        

    def print_diffusivities(self, show_on_screen=True):
        """Print out all diffusivities, including peak-specific, in profile"""
        if self.peakpos is None:
            self.get_peakfit()

        D_area_wb = ufloat(self.D_area_wb, self.D_area_wb_error)   
        wbmax = '{:.2f}'.format(self.maximum_wb_area)

        print('\n', self.name)
        print(' Diffusivities as log10(D in m2/s), errors, then max value A/A0')
        print('bulkH', D_area_wb, wbmax)

        # peak-specific
        for k in range(len(self.peakpos)):
            D_area_wb = ufloat(self.D_peakarea_wb[k], 
                               self.D_peakarea_wb_error[k])   
            awb = '{:.2f}'.format(self.peak_maximum_areas_wb[k])            
            Da_wb = '{:.2f}'.format(D_area_wb)            
            str1 = ' '.join((Da_wb, str(awb)))
            string = ' '.join((str(self.peakpos[k]), str1))
            print(string)


    def start_at_arbitrary(self, wn_matchup=3000, offset=0.):
        """For each spectrum in profile, divide raw absorbance by thickness 
        and set spectra abs_full_cm such that they overlap at the specified 
        wavenumber wn_matchup with specified offset up from zero"""
        for x in self.spectra:
            # Divide by thickness if not done already
            if x.abs_full_cm is None:
                check = x.divide_by_thickness(folder=self.folder)
                if check is False:
                    return False
            # print 'Setting to zero at wn_matchup'
            index = (np.abs(x.wn_full - wn_matchup)).argmin()
            abs_matched = (x.abs_full_cm - x.abs_full_cm[index]) + offset
            x.abs_full_cm = abs_matched
        return


class TimeSeries(Profile):
    def __init__(self, sample=None, fnames=[], time_hours=[], folder='',
                 thickness_microns=None, style_base=styles.style_points):
        self.sample = sample
        self.thickness_microns = thickness_microns
        self.fnames = fnames
        self.times_hours = time_hours
        self.style_base = style_base        
        self.folder = folder
        self.make_spectra()

    
    def plot_timeseries(self, y=None, peak_idx=None, tit=None, D_list=[], 
                        thickness_microns=None, max_hours=None,
                        style=None, idx_C0=0, figaxis=None): 
        """Plot and return figure and axis of time-series data and
        all diffusivities in D_list in m2/s
        """
        if max_hours is None:
            max_hours = max(self.times_hours)
            
        if thickness_microns is None:
            thickness_microns = self.thickness_microns
            
        if style is None:
            style = self.style_base
    
        if figaxis is None:
            fig = plt.figure()
            fig.set_size_inches(3.2, 3.2)
            ax = fig.add_subplot(111)
            fig.autofmt_xdate()
        else:
            ax = figaxis
            
        ax.set_xlabel('Time (hours)', fontsize=12)
        ax.set_ylabel('Concentration/\nMaximum Concentration', fontsize=12)
        ax.set_xlim(0, max_hours)

        # curves for diffusivities in D_list    
        for D in D_list:
             t, cc = models.diffusionThinSlab(log10D_m2s=D, 
                                        thickness_microns=thickness_microns, 
                                        max_time_hours=max_hours)
             ax.plot(t, cc, '-k', linewidth=1)
    
        # plot area data
        x = self.times_hours
        if y is None:
            if peak_idx is None:
                if len(self.areas) < 1:
                    self.make_areas()
                C = self.areas
            else:
                print('not ready for peak-specific yet')
            C0 = C[idx_C0]
            y = np.array(C) / C0
        
        ax.plot(x, y, **style)
        
        if figaxis is None:
            return fig, ax


def subtract_2spectra(list2, wn_high=4000, wn_low=3000):
    """Subtract spectrum 1 from spectrum 0 input as list between given
    wavenumbers (defaults to 4000 and 3000 cm-1). Spectra do not need 
    to be the same length or have the exact same wavenumbers."""
    # initial checks
    if len(list2) != 2:
        print('Takes a list of exactly 2 spectra')
        print('length:', len(list2))
        return
    for x in list2:
        # Check they are spectra
        if isinstance(x, Spectrum) is False:
            print(x, 'is not a Spectrum')
            return
        # Check for and if necessary make absorbance and wavenumber full range
        if x.abs_full_cm is None:
            check = x.start_at_zero()
            if check is False:
                return False
        if x.wn_full is None:
            x.make_baseline()

    # Set up wavenumber list for just the main spectrum 0
    x = list2[0]
    index_lo = (np.abs(x.wn_full-wn_low)).argmin()
    index_hi = (np.abs(x.wn_full-wn_high)).argmin()
    
    wn_upper_spectrum = x.wn_full[index_lo:index_hi]
    abs_upper_spectrum = x.abs_full_cm[index_lo:index_hi]
    abs_difference = np.zeros_like(wn_upper_spectrum)

    idx = 0
    for wn in wn_upper_spectrum:
        # find index of nearest wavenumber in spectrum to be subtracted off
        idx_subtract = (np.abs(list2[1].wn_full-wn)).argmin()
        # subtract
        abs_difference[idx] = (abs_upper_spectrum[idx] - 
                                list2[1].abs_full_cm[idx_subtract])
        idx += 1

    dx = wn_upper_spectrum[-1] - wn_upper_spectrum[0]
    dy = np.mean(abs_difference)
    area = dx * dy
    return area

    if len(list2[0].base_wn) != len(list2[1].base_wn):
        print('Length problem in subtract_spectra. To be dealt with.')
        return
        

def make_all_specta_lists(classname=Profile):
    for obj in gc.get_objects():
        if isinstance(obj, classname):
            obj.make_spectra()
    

def plotsetup_3x3minus2(yhi = 1, ylo = 0, xlo = 3000, xhi = 4000,
                        xtickgrid=250, ytickgrid=0.5):
    """Setup plot for spectra comparison e.g., Kunlun_peakcheck.py"""
    fig = plt.figure()
    fig.set_size_inches(6.5, 6.5)
    gs = gridspec.GridSpec(3,3)
    ax1 = plt.subplot(gs[0, 0])
    ax2 = plt.subplot(gs[0, 1])
    ax3 = plt.subplot(gs[0, 2])
    ax5 = plt.subplot(gs[1, 1])
    ax6 = plt.subplot(gs[1, 2])
    ax8 = plt.subplot(gs[2, 1])
    ax9 = plt.subplot(gs[2, 2])

    axis_list = [ax1, ax2, ax3, ax5, ax6, ax8, ax9]
    gs.update(bottom=0.14, left=0.15, right=0.95, top=0.95)    
    xmajorLocator = MultipleLocator(xtickgrid)
    ymajorLocator = MultipleLocator(ytickgrid)
    for ax in axis_list:
        ax.grid()
        ax.set_xlim([xhi, xlo])
        ax.set_ylim([ylo, yhi])
        ax.xaxis.set_major_locator(xmajorLocator)
        ax.yaxis.set_major_locator(ymajorLocator)
        plt.setp(ax.get_xticklabels(), visible=False)
        plt.setp(ax.get_yticklabels(), visible=False)
        ax.get_xaxis().get_major_formatter().set_useOffset(False)
    for k in range(len(axis_list)):
        axis_list[k].text(xhi-(0.02*xhi), yhi-(0.2*yhi), 
                string.ascii_uppercase[k], fontweight='bold')
    ax1.set_ylabel('Absorbance (cm$^{-1}$)\nPolarized\nBefore heating')
    ax5.set_ylabel('Absorbance (cm$^{-1}$)\nUnpolarized\nBefore heating')
    ax8.set_ylabel('Absorbance (cm$^{-1}$)\nUnpolarized\nAfter heating')
    ax8.set_xlabel('Wavenumber (cm$^{-1}$)')
    plt.setp(ax5.get_yticklabels(), visible=True)
    plt.setp(ax9.get_xticklabels(), visible=True, rotation=45)
    for ax in [ax1, ax8]:
        plt.setp(ax.get_yticklabels(), visible=True)
        plt.setp(ax.get_xticklabels(), visible=True, rotation=45)    
    return axis_list


def plotsetup_3x3(yhi = 1, ylo = 0, xlo = 3000, xhi = 4000,
                  xtickgrid=250, ytickgrid=0.5,
                  fig_size_inches=(6.5, 6.5)):
    """Setup plot for spectra comparison e.g., Kunlun_peakcheck.py"""
    fig = plt.figure()
    fig.set_size_inches(fig_size_inches)
    gs = gridspec.GridSpec(3,3)
    ax1 = plt.subplot(gs[0, 0])
    ax2 = plt.subplot(gs[0, 1])
    ax3 = plt.subplot(gs[0, 2])
    ax4 = plt.subplot(gs[1, 0])
    ax5 = plt.subplot(gs[1, 1])
    ax6 = plt.subplot(gs[1, 2])
    ax7 = plt.subplot(gs[2, 0])
    ax8 = plt.subplot(gs[2, 1])
    ax9 = plt.subplot(gs[2, 2])
    axis_list = [ax1, ax2, ax3, ax4, ax5, ax6, ax7, ax8, ax9]
    gs.update(bottom=0.14, left=0.15, right=0.95, top=0.95)    
    xmajorLocator = MultipleLocator(xtickgrid)
    ymajorLocator = MultipleLocator(ytickgrid)
    for ax in axis_list:
        ax.grid()
        ax.set_xlim([xhi, xlo])
        ax.set_ylim([ylo, yhi])
        ax.xaxis.set_major_locator(xmajorLocator)
        ax.yaxis.set_major_locator(ymajorLocator)
        plt.setp(ax.get_xticklabels(), visible=False)
        plt.setp(ax.get_yticklabels(), visible=False)
        ax.get_xaxis().get_major_formatter().set_useOffset(False)
    for k in range(len(axis_list)):
        axis_list[k].text(xhi-(0.02*xhi), yhi-(0.2*yhi), 
                string.ascii_uppercase[k], fontweight='bold')
    ax8.set_xlabel('Wavenumber (cm$^{-2}$)')
    for ax in [ax1, ax4, ax7]:
        plt.setp(ax.get_yticklabels(), visible=True)
    for ax in [ax7, ax8, ax9]:
        plt.setp(ax.get_xticklabels(), visible=True, rotation=45)    
    return axis_list


def plotsetup_3stacked(yhi=2, ylo=0, xlo = 3200, xhi = 3800,
                  xtickgrid=200, ytickgrid=0.5):
    """Setup plot for spectra comparison e.g., Kunlun_peakcheck.py"""
    fig = plt.figure()
    fig.set_size_inches(3, 6.5)
    gs = gridspec.GridSpec(3, 1)
    ax1 = plt.subplot(gs[0, 0])
    ax2 = plt.subplot(gs[1, 0])
    ax3 = plt.subplot(gs[2, 0])
    axis_list = [ax1, ax2, ax3]
    gs.update(bottom=0.14, left=0.35, right=0.95, top=0.95)    
    xmajorLocator = MultipleLocator(xtickgrid)
    ymajorLocator = MultipleLocator(ytickgrid)

    for ax in axis_list:
        ax.set_xlim([xhi, xlo])
        ax.set_ylim([ylo, yhi])
        ax.xaxis.set_major_locator(xmajorLocator)
        ax.yaxis.set_major_locator(ymajorLocator)
        ax.get_xaxis().get_major_formatter().set_useOffset(False)
        plt.setp(ax.get_xticklabels(), rotation=45)    

    ax3.set_xlabel('Wavenumber (cm$^{-1}$)')
    for ax in [ax1, ax2]:
        plt.setp(ax.get_xticklabels(), visible=False)
    return axis_list


#%% Generate 3D whole-block area and water profiles
def make_3DWB_area_profile(final_profile, 
                           initial_profile=None, 
                           initial_area_list=None, 
                           initial_area_positions_microns=None,
                           show_plot=True, top=1.2, fig_ax=None,
                           peakwn=None):
    """Take final 3D-wholeblock FTIR profile and returns
    a profile of the ratio of the two (A/Ao). A/A0 is also saved in the 
    profile attribute wb_areas.
    Requires information about initial state, so either an initial 
    profile (best) as the profile's initial_profile attribute,
    or else an initial area list passed in with its position. 
    Defaults to making a plot with max y-value 'top'.
    Note initial_area_positions_microns is assumed to start at 0 and then 
    gets shifted for the fit.
    """
    fin = final_profile
    leng = fin.set_len()

    # If whole-block areas are already made, use them. 
    # Otherwise make them.
    if (fin.wb_areas is not None) and (len(fin.wb_areas) > 0):
        wb_areas = fin.wb_areas
    else:
        print(fin.wb_areas)
        # initial checks
        if len(fin.positions_microns) == 0:
            print('Need position information')
            return False
        if fin.length_microns is None:
            check = fin.set_len()
            if check is False:
                print('Need more info to set profile length')
                return False
        if fin.length_microns is None:
            fin.set_len()
        if len(fin.areas) == 0:
            print('making area list for profile')
            fin.make_areas(show_plot=False)
    
        # What to normalize to? Priority given to self.wb_initial_profile, then
        # initial_profile passed in here, then initial area list passed in here.
        if fin.initial_profile is not None:
            
            init = fin.initial_profile
            if init.length_microns != fin.length_microns:
                print('initial and final lengths must be the same!')
                return False
            # Make sure area lists are populated
            for profile in [init, fin]:
                if len(profile.areas) == 0:
                    print(profile.name)
                    print('making area list for profile')
                    profile.make_areas(show_plot=False)
            A0 = init.areas
            positions0 = init.positions_microns           
    
        elif initial_profile is not None:
            init = initial_profile
            if isinstance(init, Profile) is False:
                print('initial_profile argument must be a pynams Profile.')
                print('Consider using initial_area_list and positions instead')
                return False
            # Make sure area lists are populated
            for profile in [init, fin]:
                if len(profile.areas) == 0:
                    print('making area list for profile')
                    profile.make_areas(show_plot=False)
            A0 = init.areas
            positions0 = init.positions_microns
    
        elif initial_area_list is not None:
            if initial_area_positions_microns is None:
                print('Need initial_area_positions_microns for initial_area_list')
                return False
            A0 = initial_area_list
            positions0 = initial_area_positions_microns
            if len(fin.areas) == 0:
                print('making area list for final profile')
                fin.make_areas(show_plot=False)
        else:
            print('Need some information about initial state')
            return False
    
        # More initial checks
        if len(fin.areas) != len(fin.positions_microns):
            print('area and position lists do not match')
            print('length areas:', len(fin.areas))
            print('length positions list:', len(fin.positions_microns))
            return False    
        if len(A0) < 1:        
            print('Nothing in initial area list')
            return False
        if len(positions0) < 1:
            print('Nothing in initial positions list')
            return False
        if len(A0) == 1:
            print('Using single point to generate initial line')
            A0.extend([A0[0], A0[0]])
            positions0.extend([0, fin.length_microns])
            
        # Use best-fit line through initial values to normalize final data
        p = np.polyfit(positions0-(leng/2.), A0, 1)
        
        normalizeto = np.polyval(p, fin.areas)
        wb_areas = fin.areas / normalizeto
         
        # Save whole-block areas as part of profile
        fin.wb_areas = wb_areas    
    
    if show_plot is True:
        if fig_ax is None:
            f, ax = styles.plot_area_profile_outline()
        else:
            ax = fig_ax
        ax.set_ylim(0, top)
        ylabelstring = 'Final area / Initial area'
        if peakwn is not None:
            print('NOT READY FOR PEAKFITTING YET')
        ax.set_ylabel(ylabelstring)
        style = fin.choose_marker_style()
        ax.plot([-leng/2.0, leng/2.0], [1, 1], **styles.style_1)
        ax.plot(fin.positions_microns - (leng/2.0), wb_areas, **style)
        if fig_ax is None:
            return wb_areas, f, ax
        else:
            return wb_areas
    else:
        return wb_areas


def make_3DWB_water_profile(final_profile, water_ppmH2O_initial=None,
                            initial_profile=None, 
                            initial_area_list=None, 
                            initial_area_positions_microns=None,
                            show_plot=True, top=1.2, fig_ax=None):
    """Take a profile and initial water content.
    Returns the whole-block water concentration profile based on
    the profile's attribute wb_areas. If wb_areas have not been made, 
    some initial profile information and various options are passed
    to make_3DWB_area_profile().
    Default makes a plot showing A/Ao and water on parasite y-axis
    """
    fin = final_profile
    init = initial_profile

    # Set initial water
    if water_ppmH2O_initial is not None:
        w0 = water_ppmH2O_initial
    else:
        if fin.sample is not None:
            if fin.sample.initial_water is not None:
                w0  = fin.sample.initial_water
        elif init is not None:
            if init.sample is not None:
                if init.sample.initial_water is not None:
                    w0 = init.sample.initial_water
        else:
            print('Need initial water content.')
            return False
    
    # Set whole-block areas
    if (fin.wb_areas is not None) and (len(fin.wb_areas) > 0):
        wb_areas = fin.wb_areas
    else:  
        wb_areas = make_3DWB_area_profile(fin, initial_profile, 
                                          initial_area_list, 
                                          initial_area_positions_microns)
    water = wb_areas * w0
    if show_plot is True:
        # Use a parasite y-axis to show water content
        fig = plt.figure()
        ax_areas = SubplotHost(fig, 1,1,1)
        fig.add_subplot(ax_areas)
        area_tick_marks = np.arange(0, 100, 0.2)
        ax_areas.set_yticks(area_tick_marks)
        ax_water = ax_areas.twin()
        ax_water.set_yticks(area_tick_marks)
        if isinstance(w0, uncertainties.Variable):
            ax_water.set_yticklabels(area_tick_marks*w0.n)
        else:
            ax_water.set_yticklabels(area_tick_marks*w0)
        ax_areas.axis["bottom"].set_label('Position ($\mu$m)')
        ax_areas.axis["left"].set_label('Final area / Initial area')
        ax_water.axis["right"].set_label('ppm H$_2$O')
        ax_water.axis["top"].major_ticklabels.set_visible(False)
        ax_water.axis["right"].major_ticklabels.set_visible(True)
        ax_areas.grid()
        ax_areas.set_ylim(0, 1.2)
        if fin.length_microns is not None:
            leng = fin.length_microns
        else:
            leng = fin.set_len()
        ax_areas.set_xlim(-leng/2.0, leng/2.0)
            
        style = fin.choose_marker_style()
        ax_areas.plot([-leng/2.0, leng/2.0], [1, 1], **styles.style_1)
        ax_areas.plot(fin.positions_microns-leng/2.0, wb_areas, **style)
        return water, fig, ax_areas
    else:
        return water